<?php
/**
 * CreatePackaging.php
 * 
 * Perform deployment packaging of PHP project
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

/**
 * Create service packaging for Windows Azure deployment class
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Stages_CreatePackaging
{
    /**
     * @var object Holds an instance of the class
     */
	private static $_oInstance		= null;
	
	private $_oToolContext 			= null;
	
    const ENUM_PACKAGE_WEB_ROLE     = "pack_web_role";
    const ENUM_PACKAGE_SERVICE      = "pack_service";
    
    const CSPACK_ERROR_MSG			= "Error";
	
    /**
     * A private constructor; prevents direct 
     * creation of this object
     * 
     * @return void
     */
    private function __construct() 
    {
    }
   
    /**
     * The singleton method
     * 
     * @return object Instance of this class
     */
    public static function getInstance() 
    {
        if ( !isset(self::$_oInstance) ) {
            $c = __CLASS__;
            self::$_oInstance = new $c;
            self::$_oInstance->_init();
        }

        return self::$_oInstance;
    }
    
    /**
     * Initialize singleton instance.
     * 
     * @return void
     */
    private function _init()
    {
		$this->_oToolContext = WACmdLineTools_Stages_Context::getInstance();
		if ( is_null($this->_oToolContext) ) {
			throw new RuntimeException("Failed to create instance of WACmdLineTools_Stages_Context" );			
		}   
					
		if ( is_null($this->_oToolContext->strServiceDir)
			|| !is_string($this->_oToolContext->strServiceDir)
			|| empty($this->_oToolContext->strServiceDir)
			|| !is_dir($this->_oToolContext->strServiceDir)
		) {
   			throw new RuntimeException(
	   			"Service directory"
	   			. " \"{$this->_oToolContext->strServiceDir}\""
	   			. " does not exist" 
   			);			
		}
    }
   	
	/**
	 * Process endpoint to this class
	 * 
     * @param enum $ePackageRequest Package based upon staging
	 * 
	 * @return boolean True if success, else False
	 */
	public function run($ePackageRequest) 
	{
        switch ($ePackageRequest) {
        	
        case self::ENUM_PACKAGE_WEB_ROLE:
            $this->_oToolContext->verbose(
            	"Packaging \"{$this->_oToolContext->strProjectName}\" ."
            );
            
			$this->_execCreateServiceDefWebRole();
            break;
            
        case self::ENUM_PACKAGE_SERVICE:
            $this->_oToolContext->verbose(
            	"Packaging \"{$this->_oToolContext->strProjectName}\" Service."
            );
            
			$this->_execCreateServicePackage();
			break;
			
        default:
            throw new LogicException(
            	"Unexpected packaging request: \"{$ePackageRequest}\""
            );
            
            break;
        }
        		
		return true;
	}
	
   	
    /**
	 * CSPack tool creates sService definition for WebRole binaries
	 * 
	 * @return boolean Returns TRUE on success, else FALSE 
	 */	
	private function _execCreateServiceDefWebRole()
	{
		$this->_oToolContext->verbose(
			"CSPack: Create Service definition for WebRole binaries.", 
			false
		);
		
		if ( !is_file($this->_oToolContext->strServiceDefFile) ) {
			throw new RuntimeException(
				"Service Definition file does not exist."
			);
		}
		//code added for optional web role creation
		if ( !is_dir($this->_oToolContext->strWebRoleDir) && (!is_null($this->_oToolContext->strSourcePath) )) {
			throw new RuntimeException(
				"WebRole directory does not exist."
			);
		}
		
		
		$strCommand  = "\"{$this->_oToolContext->strCSPackExe}\"";
		$strCommand .= " {$this->_oToolContext->strServiceDefFile}";
		if ( !is_null($this->_oToolContext->strSourcePath)) {
		
			$strCommand .= " /role:WebRole;{$this->_oToolContext->strWebRoleDir};" . WACmdLineTools_Util_Constants::RES_FILE_WEB_ROLE_ENTRY_POINT_DLL;
		
		}
		if ( !is_null($this->_oToolContext->strWorkerRoleSourceDir) || !is_null($this->_oToolContext->strHostedWebCorePath) ) {
            $strCommand .= " /role:WorkerRole;{$this->_oToolContext->strWorkerRoleDir};".WACmdLineTools_Util_Constants::RES_FILE_WORKER_ROLE_ENTRY_POINT_DLL ; // Added by Rajiv for worker role entry.
		}
		
		$strCommand .= " /allowLegacyWebRoles /copyOnly";

		
		WACmdLineTools_Util::doExecute($strCommand, $strResultMsg, $strOutputMsg);
		
		$bExecError 
			= strcmp(
				substr($strResultMsg, 0, strlen(self::CSPACK_ERROR_MSG)), 
				self::CSPACK_ERROR_MSG
			) == 0;
		
		if ( $bExecError ) {
   			throw new RuntimeException ( 
	   			"CSPack failed: \"{$strResultMsg}\""
   			);			
		}
		
		$this->_oToolContext->verbose($strResultMsg, false);
		$this->_oToolContext->verbose($strOutputMsg, false);
				
		$strServiceDefDir 
			= "{$this->_oToolContext->strServiceDir}/" 
			. WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_CSX;
			
		if (!is_dir($strServiceDefDir)) {
   			throw new RuntimeException ( 
	   			"CSPack failed to create directory \""
	   			. WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_CSX
	   			. "\""
   			);			
		}

		$strAppRootDir 
			= "{$strServiceDefDir}/"
			. WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_APPROOT;
			
		
			
		
			
			
			
			
		if (!is_dir($strAppRootDir) && (!is_null($this->_oToolContext->strSourcePath))) {
   			throw new RuntimeException ( 
	   			"CSPack failed to create directory \""
	   			. WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_CSX
	   			. "/"
	   			. WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_APPROOT
	   			. "\""
	   		);			
		}		
				
		$this->_oToolContext->strServiceDefAppRootDir = $strAppRootDir;

		return true;
	}

    /**
	 * CSPack tool creates deployment package with WebRole binaries
	 * 
	 * @return boolean Returns TRUE on success, else FALSE.
	 */	
	private function _execCreateServicePackage()
	{
		$this->_oToolContext->verbose(
			"CSPack: Create deployment package with WebRole binaries.", 
			false
		);
		
		$strProjectCfpkg = "{$this->_oToolContext->strServiceDir}/{$this->_oToolContext->strProjectName}.cspkg";
		
		if ( is_file($strProjectCfpkg) 
			&& ( !unlink($strProjectCfpkg) || is_file($strProjectCfpkg) )
		) {
			throw new RuntimeException("Failed to delete package.");
		}
		
		if ( !is_file($this->_oToolContext->strServiceDefFile) ) {
			throw new RuntimeException(
				"Service Definition file does not exist."
			);
		}
		
		 $strAppRootDir = $this->_oToolContext->getAppRootDirPath(); 
		 
		if ( !is_dir($strAppRootDir) && (!is_null($this->_oToolContext->strSourcePath))) {
			throw new RuntimeException(
				"AppRoot directory does not exist."
			);
		}
		
		$strServiceDefDir 
			= "{$this->_oToolContext->strServiceDir}/" 
			. WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_CSX;
			
		if (!is_dir($strServiceDefDir)) {
   			throw new RuntimeException ( 
	   			"CSPack failed to create directory \""
	   			. WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_CSX
	   			. "\""
   			);			
		}
				
		$strAppRootDirWorkerRole 
			= "{$strServiceDefDir}/". WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_APPROOT_WORKERROLE;	
				
		$strCommand  = "\"{$this->_oToolContext->strCSPackExe}\"";
		$strCommand .= " {$this->_oToolContext->strServiceDefFile}";
		
		if ( !is_null($this->_oToolContext->strSourcePath)) {

			$strCommand .= " /role:WebRole;{$strAppRootDir};" . WACmdLineTools_Util_Constants::RES_FILE_WEB_ROLE_ENTRY_POINT_DLL;
		}
		
		if ( !is_null($this->_oToolContext->strWorkerRoleSourceDir) || !is_null($this->_oToolContext->strHostedWebCorePath) ) {
			$strCommand .= " /role:WorkerRole;{$strAppRootDirWorkerRole};".WACmdLineTools_Util_Constants::RES_FILE_WORKER_ROLE_ENTRY_POINT_DLL;
		}
		
		$strCommand .= " /allowLegacyWebRoles /out:{$strProjectCfpkg}";
				
		WACmdLineTools_Util::doExecute($strCommand, $strResultMsg, $strOutputMsg);
		
		$bExecError 
			= strcmp(
				substr($strResultMsg, 0, strlen(self::CSPACK_ERROR_MSG)), 
				self::CSPACK_ERROR_MSG
			) == 0;
		
		if ( $bExecError ) {
   			throw new RuntimeException ( 
	   			"CSPack failed: \"{$strResultMsg}\""
   			);			
		}
		
		$this->_oToolContext->verbose($strResultMsg, false);
		$this->_oToolContext->verbose($strOutputMsg, false);
		
		$strDeployPackFile = "{$this->_oToolContext->strProjectName}." . WACmdLineTools_Util_Constants::DEPLOY_PACKAGE_EXT;
		$strDeployPackFilePath  = "{$this->_oToolContext->strServiceDir}/{$strDeployPackFile}";
		if ( !is_file($strDeployPackFilePath) ) {
			throw new RuntimeException("Failed to create \"{$strDeployPackFilePath}\"");
		}

		$strServiceCfgFile = WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG;
		$strServiceCfgFilePath = "{$this->_oToolContext->strServiceDir}/{$strServiceCfgFile}";
		if ( !is_file($strServiceCfgFilePath) ) {
			throw new RuntimeException("Failed to create \"{$strServiceCfgFilePath}\"");
		}

		return true;
	}
}
