<?php
/**
 * CreateSetup.php
 * 
 * Prepare setup of local host for performing Deployment
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

/**
 * Prepare setup of local host for performing Deployment
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Stages_CreateSetup
{
	private $_oToolContext 			= null;
	
    const ENUM_SETUP_WEB_ROLE       = "setup_web_role";
    const ENUM_SETUP_SERVICE        = "setup_service";
    
	/**
	 * Constructor
	 * 
	 * @return void
	 */
	public function __construct()
	{
		$this->_init();
   	}
   	
   	
    /**
     * Initialize instance.
     * 
     * @return void
     */
    private function _init()
    {
    	$this->_oToolContext = WACmdLineTools_Stages_Context::getInstance();
		if ( is_null($this->_oToolContext) ) {
			throw new RuntimeException(
				"Failed to create instance of WACmdLineTools_Stages_Context" 
			);			
		}
        		
		if ( $this->_oToolContext->bDoToolResRebuild ) {		
			$this->_oToolContext->verbose("Clearing Tool's Resources Directory", false);
			WACmdLineTools_Util::doCleanDir($this->_oToolContext->strResTmpDir);
		}		

    }
      	
    /**
     * Process endpoint to this class
     * 
     * @param enum $eSetupRequest Perform setup based upon staging
     * 
     * @return boolean Returns TRUE upon success, else FALSE
     */
   	public function run($eSetupRequest)
	{
        switch ($eSetupRequest) {
        case self::ENUM_SETUP_WEB_ROLE:
            $this->_oToolContext->verbose("Creating clean workspace for \"{$this->_oToolContext->strProjectName}\".");
			$this->_createTargetDir();     	        				
			$this->_createBuildDir();
            $this->_createProjectWebRoleDir();
            $this->_addSourceInWebRole();
			$this->_createProjectServiceDir();
            break;
            
        case self::ENUM_SETUP_SERVICE:
            $this->_oToolContext->verbose("Creating clean Service directory for \"{$this->_oToolContext->strProjectName}\".");
        	
            $this->_createProjectServiceDir();
            break;
            
        default:
            throw new LogicException( "Unexpected setup request: \"{$eSetupRequest}\"");
            break;
        }
        
        return true;	
	}


	/**
	 * Create Target Directory
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */
	private function _createTargetDir()
	{
		$this->_oToolContext->verbose("Creating Target directory", false); 
		
		if ( !is_dir($this->_oToolContext->strTargetDir) ) {
			WACmdLineTools_Util::doCreateDir($this->_oToolContext->strTargetDir);
		}

		return is_dir($this->_oToolContext->strTargetDir);;
	}
	
	/**
	 * Create Build Directory
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */
	private function _createBuildDir()
	{
		$this->_oToolContext->verbose("Creating Build directory", false); 
		
		WACmdLineTools_Util::doRemoveDir($this->_oToolContext->strBuildDir);
		WACmdLineTools_Util::doCreateDir($this->_oToolContext->strBuildDir);

		return is_dir($this->_oToolContext->strBuildDir);;
	}
		
	/**
	 * Create Project's WebRole directory
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */	
	private function _createProjectWebRoleDir()
	{
		$this->_oToolContext->verbose("Creating WebRole directory", false);
		
		WACmdLineTools_Util::doRemoveDir($this->_oToolContext->strWebRoleDir);
		
		// code added by rajiv for creating web role dir if php source is present
		if ( !is_null($this->_oToolContext->strSourcePath)) {
		
			WACmdLineTools_Util::doCreateDir($this->_oToolContext->strWebRoleDir);
		
		}
		
		return is_dir($this->_oToolContext->strWebRoleDir);
	}
	
	/**
	 * Create Project's Service directory
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */		
	private function _createProjectServiceDir()
	{
		$this->_oToolContext->verbose("Creating Service directory", false);
						
		$strServiceDir = $this->_oToolContext->strServiceDir;
		
		WACmdLineTools_Util::doRemoveDir($strServiceDir);
		WACmdLineTools_Util::doCreateDir($strServiceDir);

		return is_dir($strServiceDir);
	}
	
	/**
	 * Add user provided Source to WebRole directory.
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */	
	private function _addSourceInWebRole()
	{
		if ( $this->_oToolContext->bAddDefaultPhpSource ) {
			$this->_oToolContext->verbose("Using default PHP source", false);
			return true;	
		}
		
		$strSourcePath = $this->_oToolContext->strSourcePath;
		if ( is_null($strSourcePath) 
			|| !is_string($strSourcePath)
			|| empty($strSourcePath) 
		) {
			$this->_oToolContext->verbose("No PHP sources provided for WebRole", false);
			return true;
		}
		
		$this->_oToolContext->verbose("Adding PHP source into \"{$this->_oToolContext->strProjectName}\" WebRole directory", false);
		
		WACmdLineTools_Util::copyDir(
			$strSourcePath, 
			$this->_oToolContext->strWebRoleDir, 
			false
		);

		return true;	
	}
	
}