<?php
/**
 * DeployDevFabric.php
 * 
 * Deploy package and Run in Dev Fabric
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

/**
 * Deploy package and Run in Dev Fabric
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Stages_DeployDevFabric
{
    /**
     * @var object Holds an instance of the class
     */
	private static $_oInstance			= null;

	private $_oToolContext 				= null;

	private $_strServiceDefDir			= null;
	
	private $_strDeployPackFile			= null;
	private $_strDeployPackFilePath		= null;
	
	private $_strServiceCfgFile			= null;
	private $_strServiceCfgFilePath		= null;
	
	const EXE_DEV_FABRIC_SERVICE		= "DFService.exe";	
	const EXE_DEV_STORAGE_SERVICE		= "DSService.exe";

    const ENUM_DEV_FABRIC_WEB_ROLE      = "dev_fabric_web_role";
    const ENUM_DEV_FABRIC_SERVICE       = "dev_fabric_service";
	
    /**
     * A private constructor; prevents direct 
     * creation of this object
     * 
     * @return void
     */
    private function __construct() 
    {
    }
   
    /**
     * The singleton method
     * 
     * @return object Instance of this class
     */
    public static function getInstance() 
    {
        if ( !isset(self::$_oInstance) ) {
            $c = __CLASS__;
            self::$_oInstance = new $c;
            self::$_oInstance->_init();
        }

        return self::$_oInstance;
    }

    /**
     * Initialize singleton instance.
     * 
     * @return void
     */
    private function _init()
    {
		$this->_oToolContext = WACmdLineTools_Stages_Context::getInstance();
		if ( is_null($this->_oToolContext) ) {
			throw new RuntimeException( "Failed to create instance of WACmdLineTools_Stages_Context" );
		}

		$this->_strDeployPackFile = "{$this->_oToolContext->strProjectName}." . WACmdLineTools_Util_Constants::DEPLOY_PACKAGE_EXT;
		$this->_strDeployPackFilePath  = "{$this->_oToolContext->strServiceDir}/{$this->_strDeployPackFile}";

		$this->_strServiceCfgFile = WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG;
		$this->_strServiceCfgFilePath = "{$this->_oToolContext->strServiceDir}/{$this->_strServiceCfgFile}";

		$this->_strServiceDefDir 
			= "{$this->_oToolContext->strServiceDir}/" 
			. WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_CSX;
	}
	
	/**
	 * Validate if current setup is ready for deployment.
	 * 
	 * @return boolean Returns TRUE if valid for deployment, else FALSE 
	 */
	private function _isValidSetup()
	{
		if ( !is_dir($this->_oToolContext->strServiceDir) ) {
   			throw new RuntimeException(
	   			"Service directory"
	   			. " \"{$this->_oToolContext->strServiceDir}\""
	   			. " does not exist" 
	   			
   			);			
		}
		if ( !is_dir($this->_strServiceDefDir) ) {
   			throw new RuntimeException ( 
	   			"CSPack failed to Service Definition \""
	   			. WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_CSX
	   			. "\" does not exist" 
	   			
   			);			
		}

		if ( !is_file($this->_strServiceCfgFilePath) ) {
   			throw new RuntimeException(
	   			"Service Configuration file"
	   			. " \"{$this->_strServiceCfgFile}\""
	   			. " does not exist" 
	   			
   			);		
		}
		
		if ( !is_file($this->_strDeployPackFilePath) ) {
   			throw new RuntimeException(
	   			"Deployment Package"
	   			. " \"{$this->_strDeployPackFile}\""
	   			. " does not exist" 
	   			
   			);	
		}
		
		return true;
	}
	
	/**
	 * Process endpoint to this class
     * 
     * @param enum $eDevFabricRequest Request action by Dev Fabric based upon staging
     * 
	 * @return boolean True if success, else False
	 */
	public function run($eDevFabricRequest) 
	{
        switch ($eDevFabricRequest) {
        case self::ENUM_DEV_FABRIC_WEB_ROLE:
            $this->_oToolContext->verbose(
            	"Removing \"{$this->_oToolContext->strProjectName}\""
            	. " Service if deployed upon Dev Fabric."
            );

			$this->_doDevFabricCleanup();
            break;
        case self::ENUM_DEV_FABRIC_SERVICE:
            $this->_oToolContext->verbose("Deploying \"{$this->_oToolContext->strProjectName}\" Service upon Dev Fabric.");
			
			if ( $this->_isValidSetup() ) {			
				$this->_execRunInDevFabric();
			}
            break;
        default:
            throw new LogicException( "Unexpected Dev Fabric request: \"{$eDevFabricRequest}\"");
            break;
        }
                
        return true;    
	}
	
	/**
	 * Simple remove deployment from Dev Fabric for this project
	 * 
	 * @return Return TRUE if found, else FALSE
	 */
	private function _doDevFabricCleanup() 
	{
		$bFound = false;
		$iDeploymentID = 0;
		if ( $this->_execIsDeployedInDevFabric($iDeploymentID) ) {
			$this->_execRemoveDeployedFromDevFabric($iDeploymentID);
			$bFound = true;
		}

		return $bFound;
	}
	
	/**
	 * CSRun: Start Dev Fabric
	 * 
	 * @return boolean Returns TRUE if started, else FALSE 
	 */		
	private function _execStartDevFabric()
	{
		$this->_oToolContext->verbose("CSRun: Start Dev Fabric.", false);
		
		return $this->_execDevFabricControl("start");
	}
	
	/**
	 * CSRun: Restart Dev Fabric
	 * 
	 * @return boolean Returns TRUE if started, else FALSE 
	 */		
	private function _execRestartDevFabric()
	{
		$this->_oToolContext->verbose("CSRun: Restart Dev Fabric.", false);

		$this->_execDevFabricControl("shutdown");
		$this->_execDevFabricControl("start");
		
		return true;
	}
		
	/**
	 * CSRun: Control the state of the development fabric.
	 * 
	 * @param string $strState Setting requested for Dev Fabric
	 * 
	 * @return boolean Returns TRUE if started, else FALSE 
	 */		
	private function _execDevFabricControl($strState)
	{
		if ( !in_array($strState, array("start", "shutdown", "clean")) ) {
			throw new LogicException( 
				"Invalid expected state to control Dev Fabric: {$strState}"
			);
		}
		
		$strCommand  = "\"{$this->_oToolContext->strCSRunExe}\"";
		$strCommand .= " /devfabric:{$strState}";
		
		WACmdLineTools_Util::doExecute($strCommand, $strResultMsg, $strOutputMsg);
		$this->_oToolContext->verbose($strResultMsg, false);
		$this->_oToolContext->verbose($strOutputMsg, false);

		return true;		
	}
			
	/**
	 * CSRun: Determine if Deployment for this project is Running
	 * 
	 * @param integer &$iDeploymentID  OUT Deployment ID within Dev Fabric
	 * @param string  &$strEndpointURL OUT Endpoint URL of Deployment in Dev Fabric
	 * @param boolean $bCheckVerbose   IN Perform verbose if activated.
	 * 
	 * @return boolean Returns TRUE if Running, else FALSE 
	 */		
	private function _execIsDeployedInDevFabric(
		&$iDeploymentID = null,
		&$strEndpointURL = null,
		$bCheckVerbose = true
	) {
		if ( $bCheckVerbose ) {
			$this->_oToolContext->verbose(
				"CSRun: Is this Project deployed in Dev Fabric", 
				false
			);
		}
		
		$strCommand  = "\"{$this->_oToolContext->strCSRunExe}\"";
		$strCommand .= " /status";
		
		$aOutput = array();
		$strResult = exec($strCommand, $aOutput);

		$bFound = false;
		foreach ($aOutput as $strMsg) {
			if (strncmp("Deployment-Id:", $strMsg, strlen("Deployment-Id:")) == 0) {
				$strID = substr($strMsg, strlen("Deployment-Id:") + 1);
				continue;
			}
			
			if ( !is_null($strEndpointURL) 
				&& strncmp("EndPoint:", $strMsg, strlen("EndPoint:")) == 0
			) {
				$strEndpointURL= substr($strMsg, strlen("EndPoint:") + 1);
				$this->_oToolContext->endPointUrls[] = substr($strMsg, strlen("EndPoint:") + 1);
				
								
				continue;
			}
			
			if ( strncmp("Image-Location:", $strMsg, strlen("Image-Location:")) == 0 ) {
				$strDevFabricImgDir = substr($strMsg, strlen("Image-Location:") + 1);
				$strProjectImgDir 	= $this->_strServiceDefDir;
				
				if ( !is_dir($strDevFabricImgDir) ) {
					continue;
				}
				if ( !is_dir($strProjectImgDir) ) {
					continue;
				}				
				if ( WACmdLineTools_Util::isDirSame($strDevFabricImgDir, $strProjectImgDir) ) {
					$bFound = true;
					if ( !is_null($iDeploymentID) ) {
						$iDeploymentID = intval($strID);
					}
					break;
				}
				continue;
			}		
		}
		
		if ( $bCheckVerbose ) {
			$strMsg = $bFound ? "deployed ({$iDeploymentID})" : "not deployed";
			$this->_oToolContext->verbose("CSRun: This project is {$strMsg} in Dev Fabric", false);
		}

		return $bFound;		
	}

	/**
	 * CSRun: Removes the specified deployment from the development fabric.
	 * 
	 * @param integer $iDeploymentID Specified deployment id
	 * 
	 * @return boolean Returns TRUE upon success, else FALSE 
	 */	
	private function _execRemoveDeployedFromDevFabric($iDeploymentID)
	{
		$this->_oToolContext->verbose(
			"CSRun: Removing deployment {$iDeploymentID} from Dev Fabric", 
			false
		);
		
		$strCommand  = "\"{$this->_oToolContext->strCSRunExe}\"";
		$strCommand .= " /remove:{$iDeploymentID}";
		
		WACmdLineTools_Util::doExecute($strCommand, $strResultMsg, $strOutputMsg);
		$this->_oToolContext->verbose($strResultMsg, false);
		$this->_oToolContext->verbose($strOutputMsg, false);

		return true;
	}
	
	/**
	 * WMIC: Determine if Dev Storage is Started
	 * 
	 * @return boolean Returns TRUE if started, else FALSE 
	 */	
	private function _isDevStorageRunning()
	{
		$this->_oToolContext->verbose("Determine if Dev Storage is started.", false);
		
		$bRunning 
			= WACmdLineTools_Util::isProcessRunning(
				self::EXE_DEV_STORAGE_SERVICE,
				$this->_oToolContext->strResTmpDir
			);
		
		$strState = $bRunning ? "Started" : "Stopped";
		$this->_oToolContext->verbose("Dev Storage is {$strState}.", false);

		return $bRunning;	
	}
	
	/**
	 * CSRun: Start Dev Storage
	 * 
	 * @return boolean Returns TRUE if started, else FALSE 
	 */		
	private function _execStartDevStorage()
	{
		$this->_oToolContext->verbose("CSRun: Starting Dev Storage.", false);
		
		return $this->_execDevStorageControl("start");
	}
	
	/**
	 * CSRun: Restart Dev Storage
	 * 
	 * @return boolean Returns TRUE if started, else FALSE 
	 */		
	private function _execRestartDevStorage()
	{
		$this->_oToolContext->verbose("CSRun: Restart Dev Storage.", false);

		$this->_execDevStorageControl("shutdown");
		$this->_execDevStorageControl("start");
		usleep(5000);
		
		return true;
	}
		
	/**
	 * CSRun: Control Dev Storage
	 * 
	 * @param string $strState Control
	 * 
	 * @return boolean Returns TRUE if started, else FALSE 
	 */		
	private function _execDevStorageControl($strState)
	{
		if ( !in_array($strState, array("start", "shutdown")) ) {
			throw new LogicException( 
				"Invalid expected state to control Dev Storage: {$strState}"
			);
		}
		
		$strCommand  = "\"{$this->_oToolContext->strCSRunExe}\"";
		$strCommand .= " /devstore:{$strState}";
		
		WACmdLineTools_Util::doExecute($strCommand, $strResultMsg, $strOutputMsg);
		$this->_oToolContext->verbose($strResultMsg, false);
		$this->_oToolContext->verbose($strOutputMsg, false);

		return true;
	}
	
	
	/**
	 * CSRun: Run deployment package in Dev Fabric
	 * 
	 * @return boolean Returns TRUE on success, else FALSE 
	 */	
	private function _execRunInDevFabric()
	{
		$this->_oToolContext->verbose(
			"CSRun: Running deployment in Dev Fabric", 
			false
		);
		
		$this->_execStartDevFabric();
		$this->_doDevFabricCleanup();	
		
		if ( $this->_oToolContext->bStartDevStorage ) {
			if ( !$this->_isDevStorageRunning() ) {
				$this->_execStartDevStorage();
				if ( !$this->_isDevStorageRunning() ) {
					throw new RuntimeException(
						"Failed to start Dev Storage"
					);
				}
			}
		}
		
		$strCommand  = "\"{$this->_oToolContext->strCSRunExe}\"";
		$strCommand .= " {$this->_strServiceDefDir}";
		$strCommand .= " {$this->_strServiceCfgFilePath}";

		WACmdLineTools_Util::doExecute($strCommand, $strResultMsg, $strOutputMsg);
		$this->_oToolContext->verbose($strResultMsg, false);
		$this->_oToolContext->verbose($strOutputMsg, false);
		
		$iDeploymentID  = 0;
		$strEndpointURL = "";
		
		if ( $this->_execIsDeployedInDevFabric($iDeploymentID, $strEndpointURL) ) {
			$this->_oToolContext->iDeploymentID = $iDeploymentID;
			$this->_oToolContext->strEndpointURL = $strEndpointURL;
			return true;
		}
			
		return false;
	}
	
	/**
	 * Get Deployment information from Dev Fabric
	 * 
	 * @param integer &$iDeploymentID  OUT Deployment ID in Dev Fabric
	 * @param string  &$strEndpointURL OUT Endpoint URL of Deployment in Dev Fabric
	 * 
	 * @return boolean Returns TRUE if found, else FALSE
	 */
	public function getDeploymentInfo(
		&$iDeploymentID, 
		&$strEndpointURL
	) {
		return $this->_execIsDeployedInDevFabric(
			$iDeploymentID, 
			$strEndpointURL, 
			false
		);
	}
}