<?php
/**
 * Params.php
 * 
 * Gather and Validate Usage of Command-Line Parameter
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

require_once 'Exception.php';

/**
 * Gather and Validate Usage of Command-Line Parameter
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Stages_Params
{
	private $_bAdvance 									= false;
	private $_bShowUsage 								= false;
	
	public 	$aParams 									= array();
		
	private $_oToolContext								= null;
	private $_oPhpResManifest							= null;
	
	const FORMAT_HELP_4									= "  --%s, -%s\n%20s %s\n%20s %s\n\n";
	const FORMAT_HELP_3									= "  --%s, -%s\n%20s %s\n\n";

	/*
	 * The options parameter may contain the following elements: 
	 * * Individual characters (do not accept values)
	 * * Characters followed by a colon (parameter requires mixValue)
	 * * Characters followed by two colons (optional mixValue)
	 */
	
	const OPT_L_HELP							    	= "help"; 
	const OPT_S_HELP									= "h";
	const OPT_A_HELP									= "";
	const OPT_V_HELP									= "";
	const OPT_D_HELP									= "Show parameters and their descriptions";
	
	const OPT_L_VERBOSE							    	= "verbose"; 
	const OPT_S_VERBOSE									= "v";
	const OPT_A_VERBOSE									= "";
	const OPT_V_VERBOSE									= "";
	const OPT_D_VERBOSE									= "Enable verbose progression of deployment";
		
	const OPT_L_ADVANCED							    = "advanced"; 
	const OPT_S_ADVANCED								= "a";
	const OPT_A_ADVANCED								= "";
	const OPT_V_ADVANCED								= "";
	const OPT_D_ADVANCED								= "Present advanced deployment features";
	
	const OPT_L_SOURCE_PATH								= "source";
	const OPT_S_SOURCE_PATH								= "s";
	const OPT_A_SOURCE_PATH								= "::";
	const OPT_V_SOURCE_PATH								= "<dir>";
	const OPT_D_SOURCE_PATH								= "PHP Source Path (For default Web Role: Create a directory and copy helloworld.php file from downloaded php_azure_samples_zip folder to created folder and use this directory path. ";
		
	const OPT_L_TARGET_DIR								= "target";
	const OPT_S_TARGET_DIR								= "t";
	const OPT_A_TARGET_DIR								= "::";
	const OPT_V_TARGET_DIR								= "<dir>";
	const OPT_D_TARGET_DIR								= "Target Build Path";
			
	const OPT_L_PROJECT_NAME							= "project";
	const OPT_S_PROJECT_NAME							= "n";
	const OPT_A_PROJECT_NAME							= ":";
	const OPT_V_PROJECT_NAME							= "";
	const OPT_D_PROJECT_NAME							= "Project Name (Required)";

	const OPT_L_CLEAN_REBUILD							= "cleanRebuild"; 
	const OPT_S_CLEAN_REBUILD							= "f";
	const OPT_A_CLEAN_REBUILD							= "";
	const OPT_V_CLEAN_REBUILD							= "";
	const OPT_D_CLEAN_REBUILD							= "Clean rebuild of deployment package";
	
	const OPT_L_RUN_DEV_FABRIC							= "runDevFabric"; 
	const OPT_S_RUN_DEV_FABRIC							= "u";
	const OPT_A_RUN_DEV_FABRIC							= "";
	const OPT_V_RUN_DEV_FABRIC							= "";
	const OPT_D_RUN_DEV_FABRIC							= "Deploy project into Dev Fabric and launch Browser";
	
	const OPT_L_DEFAULT_DOC							    = "defaultDoc"; 
	const OPT_S_DEFAULT_DOC								= "c";
	const OPT_A_DEFAULT_DOC								= "::";
	const OPT_V_DEFAULT_DOC								= "<file>";
	const OPT_D_DEFAULT_DOC								= "WebRole Default Document, Default: index.php";
	
	const OPT_L_WEB_ROLE_INSTANCES						= "webroleInstances";
	const OPT_S_WEB_ROLE_INSTANCES						= "i";
	const OPT_A_WEB_ROLE_INSTANCES						= "::";
	const OPT_V_WEB_ROLE_INSTANCES						= "number >= 1";
	const OPT_D_WEB_ROLE_INSTANCES						= "WebRole Deployment Instances, Default: 1";

	const OPT_L_PHP_RESOURCES_DIR						= "phpRuntime";
	const OPT_S_PHP_RESOURCES_DIR						= "r";
	const OPT_A_PHP_RESOURCES_DIR						= ":";
	const OPT_V_PHP_RESOURCES_DIR						= "<dir>";
	const OPT_D_PHP_RESOURCES_DIR						= "PHP Runtime directory (Required once)";

	const OPT_L_BUILD_N_PACKAGE							= "buildFule";
	const OPT_L_BUILD									= "build";
	
	const OPT_L_PACKAGE									= "package";
	const OPT_S_PACKAGE									= "p";
	const OPT_A_PACKAGE									= "";
	const OPT_V_PACKAGE									= "";
	const OPT_D_PACKAGE									= "Package current build";
	
	const OPT_L_DEPLOY									= "deploy";
	const OPT_S_DEPLOY									= "d";
	const OPT_A_DEPLOY									= "";
	const OPT_V_DEPLOY									= "";
	const OPT_D_DEPLOY									= "Deploy currently built package to Dev Fabric";

	const OPT_L_ADD_XDRIVE								= "xdrive";
	const OPT_S_ADD_XDRIVE								= "x";
	const OPT_A_ADD_XDRIVE								= "";
	const OPT_V_ADD_XDRIVE								= "";
	const OPT_D_ADD_XDRIVE								= "Mount an X-Drive";
	
	const STR_SHOW_USAGE 
		= "\n\nUsage:
\n Using short parameters: 
    package.php -n=MyProject -r=\"C:\\Program Files\\PHP\" [-s=..\\php_source\\] [-t=..\\workspace\]
\n Using long parameters: 
    package.php --project=MyProject --phpRuntime=\"C:\\Program Files\\PHP\" [--source=..\\php_source\\] [--target=..\\workspace\\]
\n";

	const OPT_L_WORKERROLE_RESOURCES_DIR				= "worker-role";
	const OPT_S_WORKERROLE_RESOURCES_DIR				= "w";
	const OPT_A_WORKERROLE_RESOURCES_DIR				= ":";
	const OPT_V_WORKERROLE_RESOURCES_DIR				= "<dir>";
	const OPT_D_WORKERROLE_RESOURCES_DIR				= "Worker Role directory";
	
	const OPT_L_WORKERROLE_SCRIPT_PATH					= "worker-role-startup-script";
	const OPT_S_WORKERROLE_SCRIPT_PATH					= "b";
	const OPT_A_WORKERROLE_SCRIPT_PATH					= ":";
	const OPT_V_WORKERROLE_SCRIPT_PATH					= "<file>";
	const OPT_D_WORKERROLE_SCRIPT_PATH					= "Worker role startup script path";
	
	
	const OPT_L_SCHEDULE_TIME_DURATION				    = "scheduled-task-duration";
	const OPT_S_SCHEDULE_TIME_DURATION					= "q";
	const OPT_A_SCHEDULE_TIME_DURATION					= ":";
	const OPT_V_SCHEDULE_TIME_DURATION					= "";
	const OPT_D_SCHEDULE_TIME_DURATION					= "Run start up script after time duration (hh:mm:ss)";
	
	
	const OPT_L_HOSTED_WEB_CORE_PATH					= "hosted-web-core";
	const OPT_S_HOSTED_WEB_CORE_PATH					= "e";
	const OPT_A_HOSTED_WEB_CORE_PATH					= ":";
	const OPT_V_HOSTED_WEB_CORE_PATH					= "<dir>";
	const OPT_D_HOSTED_WEB_CORE_PATH					= "HWC website directory path";

	
	
	/**
	 * Constructor
	 * 
	 * @return void
	 */
	public function __construct()
	{
		$this->_init();
   	}
   	

    /**
     * Initialize singleton instance.
     * 
     * @return void
     */
    private function _init()
    {
		$this->_oToolContext = WACmdLineTools_Stages_Context::getInstance();
		if ( is_null($this->_oToolContext) ) {
			throw new RuntimeException(
				"Failed to create instance of WACmdLineTools_Stages_Context" 	
			);			
		}
    }
   	  
	/**
	 * Process endpoint to this class
	 * 
	 * @param array   $argv Arguments
	 * @param integer $argc Argument count
	 * 
	 * @return boolean True if success, else False
	 */
	public function run($argv, $argc) 
	{
		if ( 0 == $argc ) {
			$this->_oToolContext->bReport = false;
			$this->showUsage();
            exit(0);
		}
		
		if ( !$this->_getOpt() ) {
			$this->_oToolContext->bReport = false;
			$this->showUsage();
			return false;
		}
		
		if ( $this->_areHelpParams() ) {
			$this->_oToolContext->bReport = false;
			$this->showUsage();
			exit(0);
		}
					
		if ( !$this->_doValidateParams()
		) {
			$this->_oToolContext->bReport = false;
			$this->showUsageshowUsage();
			return false;
		}
		
        $this->_oToolContext->setParams($this->aParams);
		
		return true;
	}
	
	/**
	 * http://php.net/manual/en/function.getopt.php
	 * 
	 * @return unknown_type
	 */
	private function _getOpt() 
	{
        $aSNames = array();
        $aLNames = array();
        $aAttribs = array();

        $aSNames[]  = self::OPT_S_VERBOSE;
        $aLNames[]  = self::OPT_L_VERBOSE;
        $aAttribs[] = self::OPT_A_VERBOSE;

        $aSNames[]  = self::OPT_S_HELP;
        $aLNames[]  = self::OPT_L_HELP;
        $aAttribs[] = self::OPT_A_HELP;

        $aSNames[]  = self::OPT_S_ADVANCED;
        $aLNames[]  = self::OPT_L_ADVANCED;
        $aAttribs[] = self::OPT_A_ADVANCED;

        $aSNames[]  = self::OPT_S_SOURCE_PATH;
        $aLNames[]  = self::OPT_L_SOURCE_PATH;
        $aAttribs[] = self::OPT_A_SOURCE_PATH;

        $aSNames[]  = self::OPT_S_TARGET_DIR;
        $aLNames[]  = self::OPT_L_TARGET_DIR;
        $aAttribs[] = self::OPT_A_TARGET_DIR;

        $aSNames[]  = self::OPT_S_PROJECT_NAME;
        $aLNames[]  = self::OPT_L_PROJECT_NAME;
        $aAttribs[] = self::OPT_A_PROJECT_NAME;

        $aSNames[]  = self::OPT_S_CLEAN_REBUILD;
        $aLNames[]  = self::OPT_L_CLEAN_REBUILD;
        $aAttribs[] = self::OPT_A_CLEAN_REBUILD;

        $aSNames[]  = self::OPT_S_RUN_DEV_FABRIC;
        $aLNames[]  = self::OPT_L_RUN_DEV_FABRIC;
        $aAttribs[] = self::OPT_A_RUN_DEV_FABRIC;

        $aSNames[]  = self::OPT_S_DEFAULT_DOC;
        $aLNames[]  = self::OPT_L_DEFAULT_DOC;
        $aAttribs[] = self::OPT_A_DEFAULT_DOC;

        $aSNames[]  = self::OPT_S_WEB_ROLE_INSTANCES;
        $aLNames[]  = self::OPT_L_WEB_ROLE_INSTANCES;
        $aAttribs[] = self::OPT_A_WEB_ROLE_INSTANCES; 
             
        $aSNames[]  = self::OPT_S_PHP_RESOURCES_DIR;
        $aLNames[]  = self::OPT_L_PHP_RESOURCES_DIR;
        $aAttribs[] = self::OPT_A_PHP_RESOURCES_DIR;
        
        $aSNames[]  = self::OPT_S_PACKAGE;
        $aLNames[]  = self::OPT_L_PACKAGE;
        $aAttribs[] = self::OPT_A_PACKAGE; 
               
        $aSNames[]  = self::OPT_S_DEPLOY;
        $aLNames[]  = self::OPT_L_DEPLOY;
        $aAttribs[] = self::OPT_A_DEPLOY;
               
        $aSNames[]  = self::OPT_S_ADD_XDRIVE;
        $aLNames[]  = self::OPT_L_ADD_XDRIVE;
        $aAttribs[] = self::OPT_A_ADD_XDRIVE;
		
		$aSNames[]  = self::OPT_S_WORKERROLE_RESOURCES_DIR;
        $aLNames[]  = self::OPT_L_WORKERROLE_RESOURCES_DIR;
        $aAttribs[] = self::OPT_A_WORKERROLE_RESOURCES_DIR;
		
		$aSNames[]  = self::OPT_S_WORKERROLE_SCRIPT_PATH;
        $aLNames[]  = self::OPT_L_WORKERROLE_SCRIPT_PATH;
        $aAttribs[] = self::OPT_A_WORKERROLE_SCRIPT_PATH;     
		
		$aSNames[]  = self::OPT_S_SCHEDULE_TIME_DURATION;
        $aLNames[]  = self::OPT_L_SCHEDULE_TIME_DURATION;
        $aAttribs[] = self::OPT_A_SCHEDULE_TIME_DURATION;     
		
		$aSNames[]  = self::OPT_S_HOSTED_WEB_CORE_PATH;
        $aLNames[]  = self::OPT_L_HOSTED_WEB_CORE_PATH;
        $aAttribs[] = self::OPT_A_HOSTED_WEB_CORE_PATH;     
		
		 
		        
        $aParams = self::_parseArgs($aSNames, $aLNames, $aAttribs);

		if ( $aParams === false
			|| is_null($aParams)
			|| !is_array($aParams)
			|| empty($aParams)
		) {
			return false;
		}
		
		$this->aParams = $aParams;		
		return true;
	}

	/**
	 * Gets options from the command line argument list
	 * 
	 * @param array $aSNames  Array of short names
	 * @param array $aLNames  Array of long names
	 * @param array $aAttribs Array of attributes
	 * 
	 * @return mix Array of options selected or False if invalid
	 */
	private static function _parseArgs(
		$aSNames = array(), 
		$aLNames = array(), 
		$aAttribs = array()
	) {
        $aParams = array();
        $argv = $GLOBALS['argv'];
        
		$iIndex = 0;	
        
        foreach ($argv as $arg) {
        
            // --long --long=value
            if (substr($arg, 0, 2) == '--') {
                $eqPos                  = strpos($arg, '=');
                                
                if ($eqPos === false) {
                	// --long
                    $strKey = substr($arg, 2);
                    
                	if ( !WACmdLineTools_Util::arrayIndex($strKey, $aLNames) ) {
                		throw new WACmdLineTools_Stages_Params("Invalid long parameter entered: \"{$arg}\"");
                    }
                    
                    $strLName 			= $strKey;
                        
                    if ( array_key_exists($strLName, $aParams) ) {
                    	throw new WACmdLineTools_Stages_Params_Exception("Duplicate parameter entry for \"{$strLName}\"");
                    }
                    
					$mixValue           = isset($aParams[$strLName]) ? $aParams[$strLName] : true;
                    $aParams[$strLName]	= $mixValue;
                } else {
                	// --long=value
                    $strKey = substr($arg, 2, $eqPos-2);
                    
					if ( !WACmdLineTools_Util::arrayIndex($strKey, $aLNames) ) {
						throw new WACmdLineTools_Stages_Params_Exception("Invalid long parameter entered: \"{$arg}\"");
                    }
                    
                    $strLName 			= $strKey;
                        
                    if ( array_key_exists($strLName, $aParams) ) {
                    	throw new WACmdLineTools_Stages_Params_Exception("Duplicate parameter entry for \"{$strLName}\"");
                    }
                    
                    $mixValue           = substr($arg, $eqPos+1);
                    $aParams[$strLName]	= $mixValue;
                }
            } else if (substr($arg, 0, 1) == '-') {
             
                if (substr($arg, 2, 1) == '=') {
                	// -k=value
                    $strKey                = substr($arg, 1, 1);
                    
					if ( !WACmdLineTools_Util::arrayIndex($strKey, $aSNames, $iIndex) ) {
						throw new WACmdLineTools_Stages_Params_Exception("Invalid short parameter entered: \"{$arg}\"");
                    }
                    
                    $strLName 				= $aLNames[$iIndex];
                        
                    if ( array_key_exists($strLName, $aParams) ) {
                    	throw new WACmdLineTools_Stages_Params_Exception("Duplicate parameter entry for \"{$strLName}\"");
                    }
                     
                    $mixValue              	= substr($arg, 3);
                    $aParams[$strLName]		= $mixValue;
                } else { 
                	// -kkk
                    $aChars = str_split(substr($arg, 1));
                    foreach ($aChars as $strChar) {
                        $strKey = $strChar;
                        
						if ( !WACmdLineTools_Util::arrayIndex($strKey, $aSNames, $iIndex) ) {
							throw new WACmdLineTools_Stages_Params_Exception("Invalid short parameter entered: \"{$arg}\"");
	                    }
	                    
	                    $strLName 			= $aLNames[$iIndex];
                        
	                    if ( array_key_exists($strLName, $aParams) ) {
	                    	throw new WACmdLineTools_Stages_Params_Exception("Duplicate parameter entry for \"{$strLName}\"");
	                    }
	                    
                        $mixValue          	= isset($aParams[$strKey]) ? $aParams[$strKey] : true;
                        $aParams[$strLName]	= $mixValue;
                    }
                }
            } else { // plain-arg
				throw new WACmdLineTools_Stages_Params_Exception("Invalid parameter entered: \"{$arg}\"");
            }
        }
        return $aParams;
	}
	
    	
    /**
     * Show usage and parameters
     * 
     * @return void
     */
    public function showUsage() 
	{	
		$bShowAdvanced 
            = isset($this->aParams[self::OPT_L_ADVANCED]) 
                ? $this->aParams[self::OPT_L_ADVANCED] 
			    : false;
		
		$strUsage = self::STR_SHOW_USAGE;
		
		$strEnvTempValue = WACmdLineTools_Util_Constants::getTempEnvValue();
		$strTargetDirAddendum 
			= ( $strEnvTempValue === false ) 
				? ", Required" 
				: ", Default:\n\n          \"{$strEnvTempValue}\"";	
		
		$aShowUsage  
			= array(
		        array(
		            self::OPT_L_HELP,
		            self::OPT_S_HELP,
		            self::OPT_V_HELP,
		            self::OPT_D_HELP,
		        ),
		        array(
		            self::OPT_L_ADVANCED,
		            self::OPT_S_ADVANCED,
		            self::OPT_V_ADVANCED,
		            self::OPT_D_ADVANCED,
		        ),
		        array(
		            self::OPT_L_VERBOSE,
		            self::OPT_S_VERBOSE,
		            self::OPT_V_VERBOSE,
		            self::OPT_D_VERBOSE,
		        ),
		        array(
		        	self::OPT_L_PROJECT_NAME,
		            self::OPT_S_PROJECT_NAME,
		            self::OPT_V_PROJECT_NAME,
		            self::OPT_D_PROJECT_NAME,
		        ),
		        array(
		        	self::OPT_L_PHP_RESOURCES_DIR,
		            self::OPT_S_PHP_RESOURCES_DIR,
		            self::OPT_V_PHP_RESOURCES_DIR,
		            self::OPT_D_PHP_RESOURCES_DIR,
		        ),
		        array(
		            self::OPT_L_SOURCE_PATH,
		            self::OPT_S_SOURCE_PATH,
		            self::OPT_V_SOURCE_PATH,
		            self::OPT_D_SOURCE_PATH,
		        ),
		        array(
		            self::OPT_L_TARGET_DIR,
		            self::OPT_S_TARGET_DIR,
		            self::OPT_V_TARGET_DIR,
		            self::OPT_D_TARGET_DIR . $strTargetDirAddendum,
		        ),
		        array(
		            self::OPT_L_CLEAN_REBUILD,
		            self::OPT_S_CLEAN_REBUILD,
		            self::OPT_V_CLEAN_REBUILD,
		            self::OPT_D_CLEAN_REBUILD,
		        ),
		        array(
		            self::OPT_L_RUN_DEV_FABRIC,
		            self::OPT_S_RUN_DEV_FABRIC,
		            self::OPT_V_RUN_DEV_FABRIC,
		            self::OPT_D_RUN_DEV_FABRIC,
		        ),
            array(
		            self::OPT_L_DEFAULT_DOC,
		            self::OPT_S_DEFAULT_DOC,
		            self::OPT_V_DEFAULT_DOC,
		            self::OPT_D_DEFAULT_DOC,
		        )				
			);
			
		if ( $bShowAdvanced ) {
			array_push(
		        $aShowUsage,
		        array(
		            self::OPT_L_WEB_ROLE_INSTANCES,
		            self::OPT_S_WEB_ROLE_INSTANCES,
		            self::OPT_V_WEB_ROLE_INSTANCES,
		            self::OPT_D_WEB_ROLE_INSTANCES,
		        ),
		        array(
		            self::OPT_L_PACKAGE,
		            self::OPT_S_PACKAGE,
		            self::OPT_V_PACKAGE,
		            self::OPT_D_PACKAGE,
		        ),
		        array(
		            self::OPT_L_DEPLOY,
		            self::OPT_S_DEPLOY,
		            self::OPT_V_DEPLOY,
		            self::OPT_D_DEPLOY,
		        ),
		        array(
		            self::OPT_L_ADD_XDRIVE,
		            self::OPT_S_ADD_XDRIVE,
		            self::OPT_V_ADD_XDRIVE,
		            self::OPT_D_ADD_XDRIVE,
		        ),
                array(
		        	self::OPT_L_WORKERROLE_RESOURCES_DIR,
		            self::OPT_S_WORKERROLE_RESOURCES_DIR,
		            self::OPT_V_WORKERROLE_RESOURCES_DIR,
		            self::OPT_D_WORKERROLE_RESOURCES_DIR,
		        ),					
				array(
		        	self::OPT_L_WORKERROLE_SCRIPT_PATH,
		            self::OPT_S_WORKERROLE_SCRIPT_PATH,
		            self::OPT_V_WORKERROLE_SCRIPT_PATH,
		            self::OPT_D_WORKERROLE_SCRIPT_PATH,
		        ),		
				array(
		        	self::OPT_L_SCHEDULE_TIME_DURATION,
		            self::OPT_S_SCHEDULE_TIME_DURATION,
		            self::OPT_V_SCHEDULE_TIME_DURATION,
		            self::OPT_D_SCHEDULE_TIME_DURATION,
		        ),		
				array(
		        	self::OPT_L_HOSTED_WEB_CORE_PATH,
		            self::OPT_S_HOSTED_WEB_CORE_PATH,
		            self::OPT_V_HOSTED_WEB_CORE_PATH,
		            self::OPT_D_HOSTED_WEB_CORE_PATH,
		        )
			);
		}
			
		foreach ( $aShowUsage as $aUsage ) {
			$strUsage .= self::_doPrintUsage($aUsage);
		}
			
		echo $strUsage;
	}

	
	/**
	 * Validate collected parameter options.
	 * 
	 * @return boolean True if valid, else False
	 */
	private function _areHelpParams()
	{	
		/*
		 * Base Parameters
		 */
		if ( !isset($this->aParams[self::OPT_L_ADVANCED]) ) {
			$this->aParams[self::OPT_L_ADVANCED] = false;
		} else {
			$this->_bAdvance = true;
			
			if (count($this->aParams) == 1) {
				$this->_bShowUsage = true;
			}
		}
		
		if ( !isset($this->aParams[self::OPT_L_HELP]) ) {
			$this->aParams[self::OPT_L_HELP] = false;
		} else {
			$this->_bShowUsage = true;
		}

		return $this->_bShowUsage;
	}
	
	/**
	 * Validate collected parameter options.
	 * 
	 * @return boolean True if valid, else False
	 */
	private function _doValidateParams()
	{	
		/*
		 * Base Parameters
		 */
		if ( !isset($this->aParams[self::OPT_L_VERBOSE]) ) {
			$this->aParams[self::OPT_L_VERBOSE] = false;
		}
		
		$this->_oToolContext->verbose(
			"Validating parameters."
		);
				
		/*
		 * Windows Azure Deployment Params Validation
		 */

		/*
		 * PROJECT_NAME
		 */
		if ( !isset($this->aParams[self::OPT_L_PROJECT_NAME]) ) {
			throw new WACmdLineTools_Stages_Params_Exception(
				"--"
				. self::OPT_L_PROJECT_NAME
				. " (-"
				. self::OPT_S_PROJECT_NAME
				. ") "
				. "is required"
			);
			return false;
		} else {
			$strProjectName = $this->aParams[self::OPT_L_PROJECT_NAME];
			if ( !is_string($strProjectName) || empty($strProjectName) ) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_PROJECT_NAME
					. " (-"
					. self::OPT_S_PROJECT_NAME
					. ") "
					. "is required"
				);
			}
			
			$iMatches = preg_match(WACmdLineTools_Util_Constants::PROJECT_NAME_REGEXP_MATCH, $strProjectName);			
			if ( 0 == $iMatches ) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_PROJECT_NAME
					. " (-"
					. self::OPT_S_PROJECT_NAME
					. ") "
					. "provided is invalid [a-z,A-Z,0-9,-,_]{1,32}"
				);				
			}
		}

		/*
		 * PHP_RESOURCES_DIR
		 */
		if ( !isset($this->aParams[self::OPT_L_PHP_RESOURCES_DIR]) ) {
			$this->aParams[self::OPT_L_PHP_RESOURCES_DIR] = null;		
		} else {
			$mixValue = $this->aParams[self::OPT_L_PHP_RESOURCES_DIR];
			if ( !is_string($mixValue)
				|| empty($mixValue)
			) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_PHP_RESOURCES_DIR
					. " (-"
					. self::OPT_S_PHP_RESOURCES_DIR
					. ") "
					. "has an invalid value"
				);
			}
			
			$strRealPath = realpath($mixValue);
			
			if ( $strRealPath === false
				|| !is_dir($mixValue)
			) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_PHP_RESOURCES_DIR
					. " (-"
					. self::OPT_S_PHP_RESOURCES_DIR
					. ") "
					. "is an invalid directory path"
				);
			}
			
			if (WACmdLineTools_Util::getDirFileCount($strRealPath) == 0) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_PHP_RESOURCES_DIR
					. " (-"
					. self::OPT_S_PHP_RESOURCES_DIR
					. ") "
					. "is empty"
				);				
			}
			
			$this->aParams[self::OPT_L_PHP_RESOURCES_DIR] = $strRealPath;
		}

		/*
		 * SOURCE_PATH
		 */
		if ( isset($this->aParams[self::OPT_L_SOURCE_PATH]) ) {
			$mixValue = $this->aParams[self::OPT_L_SOURCE_PATH];
			if ( !is_string($mixValue)
				|| empty($mixValue)
			) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_SOURCE_PATH
					. " (-"
					. self::OPT_S_SOURCE_PATH
					. ") "
					. "has an invalid value"
				);
			}
			
			$strRealPath = realpath($mixValue);
			
			if ( $strRealPath === false
				|| !is_dir($mixValue)
			) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_SOURCE_PATH
					. " (-"
					. self::OPT_S_SOURCE_PATH
					. ") "
					. "is an invalid path"
				);
			}
			
			if (WACmdLineTools_Util::getDirFileCount($strRealPath) == 0) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_SOURCE_PATH
					. " (-"
					. self::OPT_S_SOURCE_PATH
					. ") "
					. "is empty: \"{$mixValue}\""
				);				
			}
			
			$this->aParams[self::OPT_L_SOURCE_PATH] = $strRealPath;
		} else {
			$this->aParams[self::OPT_L_SOURCE_PATH] = null;
		}
		
		/*
		 * TARGET DIRECTORY
		 */
		if ( isset($this->aParams[self::OPT_L_TARGET_DIR]) ) {
			$sTargetDir = $this->aParams[self::OPT_L_TARGET_DIR];
			if ( !is_string($sTargetDir)
				|| empty($sTargetDir)
			) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_TARGET_DIR
					. " (-"
					. self::OPT_S_TARGET_DIR
					. ") "
					. "has an invalid value"
				);
			}
			
			$iMatches = preg_match(WACmdLineTools_Util_Constants::TARGET_PATH_REGEXP_MATCH, $sTargetDir);			
			if ( 0 == $iMatches ) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_TARGET_DIR
					. " (-"
					. self::OPT_S_TARGET_DIR
					. ") "
					. "provided is an invalid naming for a directory \"{$sTargetDir}\" " . WACmdLineTools_Util_Constants::TARGET_PATH_REGEXP_MATCH
				);				
			}
						
			$strRealPath = realpath($sTargetDir);
			
			$strTargetDir = ( $strRealPath !== false ) ? $strRealPath : $sTargetDir;
		} else {
			$strTargetDir  = WACmdLineTools_Util_Constants::getTempAppRoot();	
		}
		
		$this->aParams[self::OPT_L_TARGET_DIR] 
			= rtrim(WACmdLineTools_Util::doPathTransform($strTargetDir), "\/\\ ");
		
		if ( !isset($this->aParams[self::OPT_L_CLEAN_REBUILD]) ) {
			$this->aParams[self::OPT_L_CLEAN_REBUILD] = false;
		}
		
		if ( !isset($this->aParams[self::OPT_L_RUN_DEV_FABRIC]) ) {
			$this->aParams[self::OPT_L_RUN_DEV_FABRIC] = false;
		}
		
		if ( isset($this->aParams[self::OPT_L_DEFAULT_DOC]) ) {
			$mixValue = $this->aParams[self::OPT_L_DEFAULT_DOC];
			if ( isset($this->aParams[self::OPT_L_SOURCE_PATH]) ) {
				$sDefaultDocPath = $this->aParams[self::OPT_L_SOURCE_PATH]
					. "/"
					. $mixValue;
					
				$strRealPath = realpath($sDefaultDocPath);
			} else {
				$strRealPath = realpath($mixValue);
				$sDefaultDocPath = $mixValue;
			}
			
			if ( $strRealPath === false
				|| !is_file($sDefaultDocPath)
			) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_DEFAULT_DOC
					. " (-"
					. self::OPT_L_DEFAULT_DOC
					. ") "
					. "is an invalid file"
				);
			}
		} else {
			$this->aParams[self::OPT_L_DEFAULT_DOC] = WACmdLineTools_Util_Constants::FILE_DEFAULT_DOC;
		}	
		
		if ( !isset($this->aParams[self::OPT_L_WEB_ROLE_INSTANCES]) ) {
			$this->aParams[self::OPT_L_WEB_ROLE_INSTANCES] = 1;
		} else {
			$iWebRoleInstances = intval($this->aParams[self::OPT_L_WEB_ROLE_INSTANCES]);
			if ( $iWebRoleInstances < 1 ) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. self::OPT_L_WEB_ROLE_INSTANCES
					. " (-"
					. self::OPT_S_WEB_ROLE_INSTANCES
					. ") "
					. "has invalid value"
				);
			}
			$this->aParams[self::OPT_L_WEB_ROLE_INSTANCES] = $iWebRoleInstances;
		}
		
		if ( !isset($this->aParams[self::OPT_L_ADD_XDRIVE]) ) {
			$this->aParams[self::OPT_L_ADD_XDRIVE] = false;
		}
		/*
		 * Control building phases of a project
		 */
		$this->aParams[self::OPT_L_BUILD_N_PACKAGE] = true;
		
		if ( isset($this->aParams[self::OPT_L_BUILD]) ) {
			$this->aParams[self::OPT_L_BUILD] = true;
			$this->aParams[self::OPT_L_BUILD_N_PACKAGE] = false;
		} else {
			$this->aParams[self::OPT_L_BUILD] = false;
		}
		
		if ( isset($this->aParams[self::OPT_L_PACKAGE]) ) {
			$this->aParams[self::OPT_L_BUILD] = false;
			$this->aParams[self::OPT_L_BUILD_N_PACKAGE] = false;
		} else {
			$this->aParams[self::OPT_L_PACKAGE] = false;
		}
		
		if ( !isset($this->aParams[self::OPT_L_DEPLOY]) ) {
			$this->aParams[self::OPT_L_DEPLOY] = false;
		} else {
			$this->aParams[self::OPT_L_BUILD_N_PACKAGE] = false;
		}

		return true;
	}
	
	/**
	 * Provide usage detail for a single parameter.
	 * 
	 * @param array $aUsage Usage details
	 * 
	 * @return string Parameter usage
	 */
	private static function _doPrintUsage($aUsage)
	{
		if ( empty($aUsage[2]) ) {
			return sprintf(self::FORMAT_HELP_3, $aUsage[0], $aUsage[1], "", $aUsage[3]);
		} else {
			return sprintf(self::FORMAT_HELP_4, $aUsage[0], $aUsage[1], "", $aUsage[2], "", $aUsage[3]);
		}		
	}
}

?>
