<?php
/**
 * Resources.php
 * 
 * Copy resources from Tool's resource directory
 * into project's WebRole and Service directories
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

require_once 'Exception.php';

/**
 * Copy resources from Tool's resource directory
 * into project's WebRole and Service directories
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Stages_Resources
{
	private $_oToolContext 				= null;
	
	const ENUM_RESOURCES_WEB_ROLE		= "resources_web_role";
	const ENUM_RESOURCES_SERVICE		= "resources_service";
	
	const FILE_PHP_INI					= "php.ini";
	const FILE_PHP_INI_RECOMMENDED		= "php.ini-recommended";
	
	const FILE_PHP_EXT_CURL				= "php_curl.dll";
	
	const FILE_AZURE_TEMPLATE_ZIP		= "php_azure_template.zip";
	
    const PHP_INI_EXT_ENABLE_REG_EXP    = "/^extension=%s/"; 
    const PHP_INI_EXT_DISABLE_REG_EXP   = "/^;extension=%s/"; 
     
    const PHP_INI_EXT_ENABLE            = "extension=%s";
    const PHP_INI_EXT_DISABLE           = ";extension=%s";
	const ENUM_RESOURCES_WORKER_ROLE    = "resources_worker_role";
    
    /**
     * A private constructor; prevents direct 
     * creation of this object
     * 
     * @return void
     */
    public function __construct() 
    {
    	$this->_init();
    }
    
    /**
     * Initialize singleton instance.
     * 
     * @return void
     */
    private function _init()
    {
		$this->_oToolContext = WACmdLineTools_Stages_Context::getInstance();
		if ( is_null($this->_oToolContext) ) {
			throw new RuntimeException(
				"Failed to create instance of WACmdLineTools_Stages_Context" 
			);			
		}
    }
   	
	/**
	 * Process endpoint to this class
	 * 
	 * @param enum $eResRequest Add Resources based upon staging
	 * 
	 * @return boolean Returns TRUE upon success, else FALSE
	 */
	public function run($eResRequest)
	{
		switch ($eResRequest) {
			
		case self::ENUM_RESOURCES_WEB_ROLE:
            $this->_addWebRoleResources();

        case self::ENUM_RESOURCES_SERVICE:
            $this->_addServiceResources();
			break;
            
		case self::ENUM_RESOURCES_WORKER_ROLE:
			$this->_addWorkerRoleResources();
			$this->_addHwcDlls();
			break;

        default:
            throw new LogicException(
            	"Unexpected resource request: \"{$eResRequest}\""
           	);
            break;
        }
        
        return true;
	}
	
    /**
	 * Copy Resources to WebRole directory
	 * 
	 * @return boolean Returns TRUE upon success, else FALSE 
	 */	
	private function _addWebRoleResources()
	{
	
		$this->_oToolContext->verbose("Adding resources to Web Role.");
		
		$this->_oToolContext->verbose("Adding PHP resources to WebRoot.", false);
		
		$this->_addPhpRuntimeResource();
		$this->_addWindowsAzureSdk4PHP($this->_oToolContext->strWebRoleDir);
		$this->_addWindowsAzureDiagnosticsSupport();
		$this->_addGlobalAppFile();

		//if ( $this->_oToolContext->bAddDefaultPhpSource ) {
			$this->_addDefaultPhpSource();
		//}

		$this->_addDefaultWebConfigFile();
		
		$this->_addDefaultWebRoleConfigFile();
			
		return true;
	}
	
	/**
	 * Copy Base Resources to Service directory
	 * 
	 * @return boolean Returns TRUE on success, else FALSE 
	 */	
	private function _addServiceResources()
	{
			
		$this->_oToolContext->verbose("Adding resources to Service Directory.");
	
		$strResServiceCfgFile  = "{$this->_oToolContext->strResToolDir}/";
		$strResServiceCfgFile .= WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG;
		
		$strTargetServiceCfgFile = "{$this->_oToolContext->strServiceDir}/";
		$strTargetServiceCfgFile .= WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG;
		
		
		$strResServiceDefFile 
			= "{$this->_oToolContext->strResToolDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF;

		$strTargetServiceDefFile 
			= "{$this->_oToolContext->strServiceDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF;
				
		
		if ( !is_null($this->_oToolContext->strWorkerRoleSourceDir) ||!is_null($this->_oToolContext->strHostedWebCorePath) ) {
			
			$xml = simplexml_load_file($strResServiceCfgFile);
				
						
			$appDirArr = preg_split("/\\\/", $this->_oToolContext->strWorkerRoleSourceDir);
		    $appDir=end($appDirArr);
			
			$settingPath = $xml->Role[1]->ConfigurationSettings[0]->addChild("Setting");			
			$settingPath->addAttribute('name', 'StartupScript');
			$settingPath->addAttribute('value', $appDir."\\".$this->_oToolContext->strWorkerRoleScriptPath);
			
			if ($this->_oToolContext->strWorkerScriptTimeDuration) {
			
				// This will read worker role node(0 = Web role, 1= worker role).
				$settingDuration = $xml->Role[1]->ConfigurationSettings[0]->addChild("Setting");
				$settingDuration->addAttribute('name', 'Duration');
				$settingDuration->addAttribute('value', $this->_oToolContext->strWorkerScriptTimeDuration);
			
			}
			
			if ($this->_oToolContext->strHostedWebCorePath) {
			
				$settingHwc = $xml->Role[1]->ConfigurationSettings[0]->addChild("Setting");
				$settingHwc->addAttribute('name', 'UseHostedCore');
				$settingHwc->addAttribute('value', 'true');
							
			}
			
			
			
			
			if (is_null($this->_oToolContext->strSourcePath)) {
			
				$dom = dom_import_simplexml($xml->Role[0]);
				$dom->parentNode->removeChild($dom);
			}
			
			
			if ($fp = fopen($strTargetServiceCfgFile, 'w')) {
				$dom = new DOMDocument('1.0'); 
				$dom->preserveWhiteSpace = false; 
				$dom->formatOutput = true; 
				$dom->loadXML($xml->asXML()); 
				fwrite($fp, $dom->saveXML());				
				fclose($fp);
			
			} else {
			
				throw new RuntimeException(
					"Failed to copy file \"" 
					. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG 
					. "\""
				);	
			}
			
			
			
			$xml = simplexml_load_file($strResServiceDefFile);
						
			$appDirArr = preg_split("/\\\/", $this->_oToolContext->strWorkerRoleSourceDir);
		    $appDir=end($appDirArr);
			
			$settingPath     = $xml->WorkerRole[0]->ConfigurationSettings[0]->addChild("Setting");			
			$settingPath->addAttribute('name', 'StartupScript');
			
			if ($this->_oToolContext->strWorkerScriptTimeDuration) {
			
				$settingDuration     = $xml->WorkerRole[0]->ConfigurationSettings[0]->addChild("Setting");			
				$settingDuration->addAttribute('name', 'Duration');
							
			}
			
			if ($this->_oToolContext->strHostedWebCorePath) {
			
				$settingHwc = $xml->WorkerRole[0]->ConfigurationSettings[0]->addChild("Setting");
				$settingHwc->addAttribute('name', 'UseHostedCore');
							
			}
						
			if (is_null($this->_oToolContext->strSourcePath)) {
			
				$dom = dom_import_simplexml($xml->WebRole[0]);
				$dom->parentNode->removeChild($dom);
			}
			
			if ($fp = fopen($strTargetServiceDefFile, 'w')) {
			
				$dom = new DOMDocument('1.0'); 
				$dom->preserveWhiteSpace = false; 
				$dom->formatOutput = true; 
				$dom->loadXML($xml->asXML()); 
				fwrite($fp, $dom->saveXML());
				fclose($fp);
			
			} else {
			
				throw new RuntimeException(
					"Failed to copy file \"" 
					. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG 
					. "\""
				);	
			}
		} else {
		
				$xml = simplexml_load_file($strResServiceCfgFile);
				$dom=dom_import_simplexml($xml->Role[1]);
				$dom->parentNode->removeChild($dom);
								
			if ($fp = fopen($strTargetServiceCfgFile, 'w')) {
			
					$dom = new DOMDocument('1.0'); 
					$dom->preserveWhiteSpace = false; 
					$dom->formatOutput = true; 
					$dom->loadXML($xml->asXML()); 
					fwrite($fp, $dom->saveXML());
					fclose($fp);
			
			} else {
				
					throw new RuntimeException(
						"Failed to copy service configuration file "
					);	
			}
				
				$xml = simplexml_load_file($strResServiceDefFile);
				$dom=dom_import_simplexml($xml->WorkerRole[0]);
				$dom->parentNode->removeChild($dom);
						
				
			if ($fp = fopen($strTargetServiceDefFile, 'w')) {
			
					$dom = new DOMDocument('1.0'); 
					$dom->preserveWhiteSpace = false; 
					$dom->formatOutput = true; 
					$dom->loadXML($xml->asXML()); 
					fwrite($fp, $dom->saveXML());
					fclose($fp);
			
			} else {
				
					throw new RuntimeException(
						"Failed to copy service defination file "
					);	
			}
				
				/*
			if ( !is_file($strResServiceCfgFile)
				|| !copy($strResServiceCfgFile, $strTargetServiceCfgFile)
				|| !is_file($strTargetServiceCfgFile)
			) {
				throw new RuntimeException(
					"Failed to copy file \"" 
					. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG 
					. "\""
				);	
			} 
			
			if ( !is_file($strResServiceDefFile)
				|| !copy($strResServiceDefFile, $strTargetServiceDefFile)
				|| !is_file($strTargetServiceDefFile)
			) {
				throw new RuntimeException( 
					"Failed to copy file \"" 
					. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF 
					. "\""
				);	
			} 
			
			*/
			
		}

		return true;
	}
	

   	/**
   	 * Extract and Add Windows Azure SDK for PHP Developers
   	 * 
	 * @param string $strTargetDir Windows Azure SDK for PHP extraction dir.
	 *
   	 * @return boolean Returns TRUE on success, else FALSE
   	 */
   	private function _addWindowsAzureSdk4PHP($strTargetDir)
   	{	   		
   		$this->_oToolContext->verbose(
   			"Adding "
   		    . "\"Windows Azure SDK for PHP\" resource"
   			. " into \"{$this->_oToolContext->strProjectName}\" project.",
   			false
   		);
   		
   		$strZipFile = WACmdLineTools_Util_Constants::FILE_PHP_AZURE_SDK_ZIP;
   		$strZipFilePath = "{$this->_oToolContext->strResPhpDir}/{$strZipFile}";
   		
   		if (!is_file($strZipFilePath)) {
   			throw new WACmdLineTools_Stages_Resources_Exception ( 
	   			"ZIP file \"{$strZipFile}\" does not exist within Tool's \\res\\php\\ directory.",
   				WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED   				
   			);
   		}
   		
   		$strTmpDir = "{$strTargetDir}/_tmp";
   		$strExtractPath = WACmdLineTools_Util_Constants::ZIP_PATH_PHP_AZURE_SDK_LIB;
   		$strTmpContents = "{$strTmpDir}/{$strExtractPath}/";
   		
   		if ( !WACmdLineTools_Util_Zip::zipExtractFolder($strZipFilePath, $strTmpDir, $strExtractPath)
   			|| !is_dir($strTmpContents)
   		) {
   			throw new RuntimeException(
   				"Failed to extract"
   				. " folder \"{$strExtractPath}\""
   				. " from ZIP file \"{$strZipFile}\"" 
   			);
   		}
   				
		$strPHPAzureDir = "{$strTargetDir}/Microsoft";
   		if (!rename($strTmpContents, $strPHPAzureDir)
   			|| !is_dir($strPHPAzureDir)
   		) {
   			throw new RuntimeException( 
	   			"Failed to"
	   			. " rename \"_tmp\" directory"
	   			. " to \"Microsoft\" directory" 
   			);
   		}
   		   		
   		if (is_dir($strTmpDir)) {
   			WACmdLineTools_Util::doRemoveDir($strTmpDir);
   		}
   		
    	// $this->_updatePhpIniFile(self::FILE_PHP_EXT_CURL);

   		return true;
   	}

    /**
   	 * Add binaries needed for Windows Azure Diagnostics
   	 * 
   	 * @return void
   	 */
   	private function _addWindowsAzureDiagnosticsSupport()
    {
        $this->_oToolContext->verbose(
   			"Adding "
   		    . "\"Windows Azure Diagnostics Support\""
   			. " into \"{$this->_oToolContext->strProjectName}\" project.",
   			false
   		);

        // Create bin folder
		$strBinDir = "{$this->_oToolContext->strWebRoleDir}/bin";
		if (!is_dir($strBinDir)) {
			mkdir($strBinDir);
		}

        $strWebRoleEntryPointDLL 
			= "{$this->_oToolContext->strResToolDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_ROLE_ENTRY_POINT_DLL;

		$strTargetWebRoleEntryPointDLL 
			= "{$strBinDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_ROLE_ENTRY_POINT_DLL;
	
        // Copy Entry Point dll
		if ( !is_file($strWebRoleEntryPointDLL)
			|| !copy($strWebRoleEntryPointDLL, $strTargetWebRoleEntryPointDLL)
			|| !is_file($strTargetWebRoleEntryPointDLL)
		) {
			throw new RuntimeException( 
				"Failed to add \"" 
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_ROLE_ENTRY_POINT_DLL
				. "\" to WebRoot directory." 
			);
		}

        // Copy dependent dlls and xmls from Windows Azure SDK
        copy(
        	"{$this->_oToolContext->strWindowsAzureSdkPath}/"
            . WACmdLineTools_Util_Constants::DIR_WA_SDK_DLL . "/"
            . WACmdLineTools_Util_Constants::FILE_WA_SDK_DIAGNOSTICS_DLL,
            "{$strBinDir}/" . WACmdLineTools_Util_Constants::FILE_WA_SDK_DIAGNOSTICS_DLL
        );
        copy(
        	"{$this->_oToolContext->strWindowsAzureSdkPath}/"
            . WACmdLineTools_Util_Constants::DIR_WA_SDK_DLL . "/"
            . WACmdLineTools_Util_Constants::FILE_WA_SDK_DIAGNOSTICS_XML,
            "{$strBinDir}/" . WACmdLineTools_Util_Constants::FILE_WA_SDK_DIAGNOSTICS_XML
        );
            
        copy(
        	"{$this->_oToolContext->strWindowsAzureSdkPath}/"
            . WACmdLineTools_Util_Constants::DIR_WA_SDK_DLL . "/"
            . WACmdLineTools_Util_Constants::FILE_WA_SDK_STORAGE_CLIENT_DLL,
            "{$strBinDir}/" . WACmdLineTools_Util_Constants::FILE_WA_SDK_STORAGE_CLIENT_DLL
        );
        copy(
        	"{$this->_oToolContext->strWindowsAzureSdkPath}/"
            . WACmdLineTools_Util_Constants::DIR_WA_SDK_DLL . "/"
            . WACmdLineTools_Util_Constants::FILE_WA_SDK_STORAGE_CLIENT_XML,
            "{$strBinDir}/" . WACmdLineTools_Util_Constants::FILE_WA_SDK_STORAGE_CLIENT_XML
        );
            
        copy(
        	"{$this->_oToolContext->strWindowsAzureSdkPath}/"
            . WACmdLineTools_Util_Constants::DIR_WA_SDK_DLL . "/"
            . WACmdLineTools_Util_Constants::FILE_WA_SDK_CLOUD_DRIVE_DLL,
            "{$strBinDir}/" . WACmdLineTools_Util_Constants::FILE_WA_SDK_CLOUD_DRIVE_DLL
        );
        copy(
        	"{$this->_oToolContext->strWindowsAzureSdkPath}/"
            . WACmdLineTools_Util_Constants::DIR_WA_SDK_DLL . "/"
            . WACmdLineTools_Util_Constants::FILE_WA_SDK_CLOUD_DRIVE_XML,
            "{$strBinDir}/" . WACmdLineTools_Util_Constants::FILE_WA_SDK_CLOUD_DRIVE_XML
        );
    }
    
    /**
   	 * Add Global.asax file
   	 * 
   	 * @return void
   	 */
   	private function _addGlobalAppFile()
    {
    	$strTargetGlobalAppFile 
			= "{$this->_oToolContext->strWebRoleDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_GLOBAL_APP_FILE;
					
		if (is_file($strTargetGlobalAppFile)) {
	   		$this->_oToolContext->verbose(
	   			"User provided " 
				. WACmdLineTools_Util_Constants::RES_FILE_GLOBAL_APP_FILE 
				. " with \"{$this->_oToolContext->strProjectName}\" project.",
	   			false
	   		); 
			return;
		}
		
		$this->_oToolContext->verbose(
			"Adding default " 
			. WACmdLineTools_Util_Constants::RES_FILE_GLOBAL_APP_FILE 
			. " to WebRoot.", 
			false
		);
					
		$strResGlobalAppFile 
			= "{$this->_oToolContext->strResToolDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_GLOBAL_APP_FILE;
							
		if ( !is_file($strResGlobalAppFile)
			|| !copy($strResGlobalAppFile, $strTargetGlobalAppFile)
			|| !is_file($strTargetGlobalAppFile)
		) {
			throw new RuntimeException( 
				"Failed to add \"" 
				. WACmdLineTools_Util_Constants::RES_FILE_GLOBAL_APP_FILE 
				. "\" to WebRoot directory." 
			);
		}
    }

   	/**
   	 * Copy available PHP Resources from tool's resource directory.
   	 * 
   	 * @return boolean Returns TRUE on success, else FALSE
   	 */
   	private function _addPhpRuntimeResource()
   	{
   		$this->_oToolContext->strWebRolePhpDir 
   			= "{$this->_oToolContext->strWebRoleDir}/php";
   			
   		if ( is_dir($this->_oToolContext->strWebRolePhpDir) ) {
	   		return;  			
   		}

   		$this->_oToolContext->verbose(
   			"Adding PHP Runtime"
   			. " into \"{$this->_oToolContext->strProjectName}\" project.",
   			false
   		);
   		
		WACmdLineTools_Util::doCopyDir(
			$this->_oToolContext->strResPhpRuntimeDir, 
			$this->_oToolContext->strWebRolePhpDir,
			true
		);
			
		if ( !is_dir($this->_oToolContext->strWebRolePhpDir) ) {
			throw new RuntimeException( 
	   			"Failed to add PHP Binary directory"
	   			. " into \"{$this->_oToolContext->strProjectName}\" project."
			);
		}
   		
   		return true;
   	}

	/**
	 * Extract file from a ZIP file
	 * 
	 * @param string $strZipFile	     ZIP file path.
	 * @param string $strExtractPath	 Extraction path in ZIP file
	 * @param string $strExtractContents File to Extract from ZIP file
	 * @param string $strTargetDir	     Target directory for extracted File
	 * @param string $strTargetFile      Target file name for extracted File.
	 * 
	 * @return boolean Returns TRUE on success, else False.
	 */
	private static function _doExtractFromZipFile(
		$strZipFile,
		$strExtractPath,  
		$strExtractContents, 
		$strTargetDir,
		$strTargetFile
	) {
		$strExtractPath = !empty($strExtractPath) ? "{$strExtractPath}/" : "";
		$strExtractPath .= $strExtractContents;
		
		$strTargetDir = rtrim($strTargetDir, "\/\\ ");
		$strTargetTmpDir = "{$strTargetDir}/_tmp";
		
		WACmdLineTools_Util::doRemoveDir($strTargetTmpDir);
		mkdir($strTargetTmpDir);	
		
		$strTargetTmpFile = "{$strTargetTmpDir}/{$strExtractPath}";
			
   		if ( !WACmdLineTools_Util_Zip::zipExtractContents($strZipFile, $strTargetTmpDir, $strExtractPath)
   			|| !is_file($strTargetTmpFile)
   		) {
   			throw new RuntimeException(
   				"Failed extract file \"{$strExtractPath}\""
   				. " from ZIP file \"{$strZipFile}\""
   				. " to \"{$strTargetTmpDir}\"" 
   			);
   		}
   		
   		chmod($strTargetTmpDir, 0777);
		$strTargetPath = "{$strTargetDir}/{$strTargetFile}";
   		
		if ( is_file($strTargetPath) ) {
			unlink($strTargetPath);
		}
		if ( !rename($strTargetTmpFile, $strTargetPath)
			|| !is_file($strTargetPath)
		) {
   			throw new RuntimeException(
   				"Failed to rename file \"{$strTargetTmpFile}\"" 
   			);				
		}
		
		if ( is_dir($strTargetTmpDir) ) { 
			WACmdLineTools_Util::doRemoveDir($strTargetTmpDir);					
		}
		
		return true;
	}
	
	
	/**
	 * Extract and Validate File into PHP directory.
	 * 
	 * @param string  $strTargetPhpDir    Target PHP directory path
	 * @param string  $strZipFile		  ZIP file
	 * @param string  $strExtractPath	  Extraction path within ZIP file
	 * @param string  $strExtractContents ZIP contents to extract
	 * @param string  $strTargetFileName  Target name
	 * @param boolean $bAddPhpExt  		  Add PHP Ext
	 * @param boolean $bAddPhpIniExt  	  Add PHP Ext to php.ini
	 * 
	 * @return boolean Returns TRUE on success, else FALSE.
	 */
	private function _doExtractResource(
		$strTargetPhpDir,
		$strZipFile,
		$strExtractPath,
		$strExtractContents,
		$strTargetFileName,
		$bAddPhpExt = false,
		$bAddPhpIniExt = false
	) {	
   		$this->_oToolContext->verbose(
   			"Extract Resource: {$strZipFile}",
   			false
   		);
   		
		$strZipFilePath  = "{$this->_oToolContext->strResPhpDir}/";
	   	$strZipFilePath .= $strZipFile;
	   	  		
   		if (!is_file($strZipFilePath)) {
   			throw new WACmdLineTools_Stages_Resources_Exception ( 
	   			"ZIP file \"{$strZipFile}\" does not exist" 
   			);
   		}  

		$strTargetPhpPath  = "{$strTargetPhpDir}/";
		$strTargetPhpPath .= $bAddPhpExt ? "ext/" : "";
		
		$this->_doExtractFromZipFile(
			$strZipFilePath,
			$strExtractPath,
			$strExtractContents,
			$strTargetPhpPath,
			$strTargetFileName
		);
		
		$strTargetFile  = "{$strTargetPhpPath}/";
		$strTargetFile .= $strTargetFileName;

		if (!is_file($strTargetFile) ) {
   			throw new WACmdLineTools_Stages_Resources_Exception ( 
	   			"ZIP file \"{$strTargetFile}\" does not exist" 
   			);
   		}

   		if ($bAddPhpExt && $bAddPhpIniExt) {
   			$strPhpIniPath  = "{$strTargetPhpDir}/";
   			$strPhpIniPath .= PhpResManifest::FILE_PHP_INI;
   			
   			$strRegExp = sprintf(PhpResManifest::PHP_INI_EXT_REG_EXP, $strTargetFileName);
   			$strReplace = sprintf(PhpResManifest::PHP_INI_EXT_ENABLE, $strTargetFileName);
   			if ( !WACmdLineTools_Util::replaceLine($strPhpIniPath, $strRegExp, $strReplace) ) {
	   			throw new WACmdLineTools_Stages_Resources_Exception ( 
		   			"Failed to replace "
		   			. " \"{$strReplace}\""
		   			. " within " . PhpResManifest::FILE_PHP_INI . "" 
	   			);   				
   			}		
   		}
	}

	/**
	 * Add Default PHP Source is none is provided by User.
	 * 
	 * @return void
	 */
	private function _addDefaultPhpSource() 
	{
		$this->_oToolContext->verbose(
			"Adding default PHP sources to WebRoot.", 
			false
		);
		
		$strTargetDefaultDoc 
			= "{$this->_oToolContext->strWebRoleDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_INDEX_PHP;
		
		if (!is_file($strTargetDefaultDoc)) {	
			$strResDefaultDoc 
				= "{$this->_oToolContext->strResToolDir}/" 
					. WACmdLineTools_Util_Constants::RES_FILE_INDEX_PHP;
			
			if ( !is_file($strResDefaultDoc)
				|| !copy($strResDefaultDoc, $strTargetDefaultDoc)
				|| !is_file($strTargetDefaultDoc)
			) {
				throw new RuntimeException( 
					"Failed to add \"" 
					. WACmdLineTools_Util_Constants::RES_FILE_INDEX_PHP 
					. "\" to WebRoot directory." 
				);
			}
		}	
	}
	
	/**
	 * Add Web.config file if none is provided by user.
	 * 
	 * @return void
	 */
	private function _addDefaultWebConfigFile()
	{
		$strResWebConfig 
			= "{$this->_oToolContext->strResToolDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG;
		
		$strTargetWebConfig 
			= "{$this->_oToolContext->strWebRoleDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG;
				
		if (is_file($strTargetWebConfig)) {
		
			$xml = simplexml_load_file($strTargetWebConfig);

			$nodeArr=array();

			foreach ($xml->children() as $child) {
				$nodeArr[]=$child->getName();
			}
		
			if (count($nodeArr)==0) {
				copy($strResWebConfig, $strTargetWebConfig);
			} else {
			    if (!in_array('system.diagnostics', $nodeArr)) { 	
				    $system=$xml->addChild('system.diagnostics');
			
				    $add=$system->addChild('trace')->addChild('listeners')->addChild('add');
				    $add->addAttribute('type', "Microsoft.WindowsAzure.Diagnostics.DiagnosticMonitorTraceListener, Microsoft.WindowsAzure.Diagnostics, Version=1.0.0.0, Culture=neutral, PublicKeyToken=31bf3856ad364e35");
				    $add->addAttribute('name', 'AzureDiagnostics');

				    $filter=$add->addChild('filter');
				    $filter->addAttribute('type', "");
				   				
			    }
					
			    if (!in_array('system.webServer', $nodeArr)) {	
				    $xml=$this->_addFastCgiInWebConfigFile($xml);
				
		        } else if (in_array('system.webServer', $nodeArr)) {	
				    try {
						$xmlDefaultDocument = $xml->xpath("system.webServer/handlers/clear/add");						
				    } catch(Exception $e) 
					    {
						 $xmlDefaultDocument = $xml->xpath("system.webServer/handlers/add");						
					}
					
					if (count($xmlDefaultDocument)==0) {
						$xmlDefaultDocument = $xml->xpath("system.webServer/handlers/add");
					}
				
					if ($xmlDefaultDocument[0]['path']!="*.php") {
						$data=$xml->asXML();
						$doc=new SimpleXMLElement($data);
						foreach ($doc->children()  as $child) {
							$childName=$child->getName();

							if ($childName=='system.webServer') {
							
								$dom=dom_import_simplexml($child);
								$dom->parentNode->removeChild($dom);
								$xml=$this->_addFastCgiInWebConfigFile($doc);								
							}
						}
					}
			    }
			
			    $dom = new DOMDocument('1.0'); 
			    $dom->preserveWhiteSpace = false; 
			    $dom->formatOutput = true; 
			    $dom->loadXML($xml->asXML()); 
			    $dom->saveXML();
			    $fp = fopen($strTargetWebConfig, 'w');
			    fwrite($fp, $dom->saveXML());				
			    fclose($fp);			
		    }

   		    $this->_oToolContext->verbose(
	            "User provided " 
                . WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG 
                . " with \"{$this->_oToolContext->strProjectName}\" project.",
                false
	        ); 
			return;
		}

		$this->_oToolContext->verbose(
			"Adding default " 
			. WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG 
			. " to WebRoot.", 
			false
		);
		
		if ( !is_file($strResWebConfig)
			|| !copy($strResWebConfig, $strTargetWebConfig)
			|| !is_file($strTargetWebConfig)
		) {
				throw new RuntimeException( 
				"Failed to add \"" 
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG 
				. "\" to WebRoot directory." 
			);
		}
		if (strtolower($this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEFAULT_DOC])!='index.php') {
		    try {	
				$xml = simplexml_load_file($strTargetWebConfig);
						
				try {
					$xmlDefaultDocument = $xml->xpath("system.webServer/defaultDocument/files/clear/add");
				} catch(Exception $e) {
					$xmlDefaultDocument = $xml->xpath("system.webServer/defaultDocument/files/add");							
			    }
						
                if (count($xmlDefaultDocument)==0) {
		  			$xmlDefaultDocument = $xml->xpath("system.webServer/defaultDocument/files/add");
                }
						
						$xmlDefaultDocument[0]["value"]=$this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEFAULT_DOC];					
		        if ($fp = fopen($strTargetWebConfig, 'w')) {
							fwrite($fp, $xml->asXML());				
							fclose($fp);			
			    }
		    } catch(Exception $ex) {
					throw new RuntimeException ("Failed to load XML File \""
					. WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG
					. "\""				
					);
			}
				
	    }
	}
	
	/**
	 * Add Default Web.roleconfig file if none is provided by user.
	 *  
	 * @return void
	 */
	private function _addDefaultWebRoleConfigFile()
	{		
		$strTargetWebRoleConfig 
			= "{$this->_oToolContext->strWebRoleDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_ROLE_CONFIG;
		
		if (is_file($strTargetWebRoleConfig)) {
	   		$this->_oToolContext->verbose(
	   			"User provided " 
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_ROLE_CONFIG 
				. " with \"{$this->_oToolContext->strProjectName}\" project.",
	   			false
	   		); 
			return;
		}
		
		$this->_oToolContext->verbose(
			"Adding default " 
			. WACmdLineTools_Util_Constants::RES_FILE_WEB_ROLE_CONFIG 
			. " to WebRoot.", 
			false
		);
				
		$strResWebRoleConfig 
			= "{$this->_oToolContext->strResToolDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_ROLE_CONFIG;
			
		if ( !is_file($strResWebRoleConfig)
			|| !copy($strResWebRoleConfig, $strTargetWebRoleConfig)
			|| !is_file($strTargetWebRoleConfig)
		) {
			throw new RuntimeException( 
				"Failed to add \"" 
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_ROLE_CONFIG 
				. "\" to WebRoot directory." 
			);
		}	
	}
	
	/**
	 * Update php.ini file to enable PHP Extension.
	 * 
	 * @param string $strPhpExtension PHP Extension full file name
	 * 
	 * @return void
	 */
	private function _updatePhpIniFile($strPhpExtension)
	{
  		$strRegExp = sprintf(self::PHP_INI_EXT_ENABLE_REG_EXP, $strPhpExtension);
    	    	
   		if ( WACmdLineTools_Util::matchLine($this->_oToolContext->strWebRolePhpIniFile, $strRegExp) ) {
   			return;
   		}
   		
  		$strRegExp = sprintf(self::PHP_INI_EXT_DISABLE_REG_EXP, $strPhpExtension);
   		$strReplace = sprintf(self::PHP_INI_EXT_ENABLE, $strPhpExtension);
   		
   		if ( !WACmdLineTools_Util::replaceLine($this->_oToolContext->strWebRolePhpIniFile, $strRegExp, $strReplace) ) {
   			throw new RuntimeException ( 
	   			"Failed to replace "
	   			. " \"{$strReplace}\""
	   			. " within " . PhpResManifest::FILE_PHP_INI . "" 
   			);   				
   		}		
	}
	
	/**
	 * Copy Resources to WorkerRole directory
	 * 
	 * @return boolean Returns TRUE upon success, else FALSE 
	 */	
	private function _addWorkerRoleResources()
	{
	
		$this->_oToolContext->verbose("Creating Worker Role Directory.");
		if (!is_dir($this->_oToolContext->strWorkerRoleDir)) {
			mkdir($this->_oToolContext->strWorkerRoleDir);
		}
		$this->_addPhpToWorkerRole();	
		$this->_addWindowsAzureSdk4PHP($this->_oToolContext->strWorkerRoleDir);
		if (!is_null($this->_oToolContext->strWorkerRoleSourceDir) ) {
			$this->_addWorkerRoleSourceDir();		
		}
	
		$this->_addWorkerRoleDlls();
		
		return true;	
	}
	
	/**
   	 * Add PHP binaries needed to worker role directory
   	 * 
   	 * @return true
   	 */
	private function _addPhpToWorkerRole() 
	{
		$this->_oToolContext->strWorkerRolePhpDir 
   			= "{$this->_oToolContext->strWorkerRoleDir}/php";
   			
   		if (is_dir($this->_oToolContext->strWorkerRolePhpDir) ) {
	   		return;  			
   		}

   		$this->_oToolContext->verbose(
   			"Adding PHP to Worker Role"
   			. " into \"{$this->_oToolContext->strProjectName}\" project.",
   			false
   		);
   		
		WACmdLineTools_Util::doCopyDir(
			$this->_oToolContext->strResPhpRuntimeDir, 
			$this->_oToolContext->strWorkerRolePhpDir,
			true
		);
			
		if ( !is_dir($this->_oToolContext->strWorkerRolePhpDir) ) {
			throw new RuntimeException( 
	   			"Failed to add PHP Binary directory"
	   			. " into \"{$this->_oToolContext->strProjectName}\" project."
			);
		}
		 
		return true;
    }
	 
	/**
   	 * Add Worker Role application to the target directory
   	 * 
   	 * @return true
   	 */
	private function _addWorkerRoleSourceDir()
	{
		$targetDirArr = preg_split("/\\\/", $this->_oToolContext->strWorkerRoleSourceDir);
		$targetDir = $this->_oToolContext->strWorkerRoleDir."\\".end($targetDirArr);
		  
		WACmdLineTools_Util::copyDir(
			$this->_oToolContext->strWorkerRoleSourceDir, 
			$targetDir,
			false
		);
		return true;
	}
	 
	/**
   	 * Add binaries needed for Windows Azure Diagnostics
   	 * 
   	 * @return void
   	 */
   	private function _addWorkerRoleDlls()
    {
        $this->_oToolContext->verbose(
   			"Adding "
   		    . "\"Windows Azure Diagnostics Support for Worker Role\""
   			. " into \"{$this->_oToolContext->strProjectName}\" project.",
   			false
   		);
		
		$strResServiceCfgFile  = "{$this->_oToolContext->strResToolDir}/";	
		$strBinDir = "{$this->_oToolContext->strWorkerRoleDir}";
		

        $strWebRoleEntryPointDLL 
			= "{$this->_oToolContext->strResToolDir}/" 
			    . WACmdLineTools_Util_Constants::RES_FILE_WORKER_ROLE_ENTRY_POINT_DLL;

		$strTargetWebRoleEntryPointDLL 
			= "{$strBinDir}/" 
				. WACmdLineTools_Util_Constants::RES_FILE_WORKER_ROLE_ENTRY_POINT_DLL;
	
        // Copy Entry Point dll
		if ( !is_file($strWebRoleEntryPointDLL)
			|| !copy($strWebRoleEntryPointDLL, $strTargetWebRoleEntryPointDLL)
			|| !is_file($strTargetWebRoleEntryPointDLL)
		) {
			throw new RuntimeException( 
				"Failed to add \"" 
				. WACmdLineTools_Util_Constants::RES_FILE_WORKER_ROLE_ENTRY_POINT_DLL
				. "\" to WebRoot directory." 
			);
		}
		
		
        // Copy dependent dlls and xmls from Windows Azure SDK
		$processor_arch = getenv("PROCESSOR_ARCHITECTURE");

		if ($processor_arch != WACmdLineTools_Util_Constants::PROC_ARCH_x86)
		{
			$processor_arch = WACmdLineTools_Util_Constants::PROC_ARCH_x64;
		}
		
        copy(
        	"{$this->_oToolContext->strWindowsAzureSdkPath}/"
			. WACmdLineTools_Util_Constants::DIR_WA_SDK_BIN_RUNTIMES_BASE . "/"
			. $processor_arch  . "/"
            . WACmdLineTools_Util_Constants::WORKER_ROLE_ROLES_MSS_DLL,
            "{$strBinDir}/" . WACmdLineTools_Util_Constants::WORKER_ROLE_ROLES_MSS_DLL
        );
		
		
		copy(
        	"{$this->_oToolContext->strWindowsAzureSdkPath}/"
            . WACmdLineTools_Util_Constants::DIR_WA_SDK_DLL . "/"
            . WACmdLineTools_Util_Constants::FILE_WA_SDK_DIAGNOSTICS_DLL,
            "{$strBinDir}/" . WACmdLineTools_Util_Constants::FILE_WA_SDK_DIAGNOSTICS_DLL
        );
        
            
        copy(
        	"{$this->_oToolContext->strWindowsAzureSdkPath}/"
            . WACmdLineTools_Util_Constants::DIR_WA_SDK_DLL . "/"
            . WACmdLineTools_Util_Constants::FILE_WA_SDK_STORAGE_CLIENT_DLL,
            "{$strBinDir}/" . WACmdLineTools_Util_Constants::FILE_WA_SDK_STORAGE_CLIENT_DLL
        );      
        
		copy(
        	"{$this->_oToolContext->strWindowsAzureSdkPath}/"
            . WACmdLineTools_Util_Constants::DIR_WA_SDK_DLL . "/"
            . WACmdLineTools_Util_Constants::FILE_WA_SDK_CLOUD_DRIVE_DLL,
            "{$strBinDir}/" . WACmdLineTools_Util_Constants::FILE_WA_SDK_CLOUD_DRIVE_DLL
        );        		
    }

	/**
   	 * Add fast cgi and default document tag in web config file
   	 * 
   	 * @param string $xml xml of web.config file
   	 * 
   	 * @return string $xml
   	 */
	private function _addFastCgiInWebConfigFile($xml)
	{
		$sysWeb = $xml->addChild('system.webServer');
		$handler  = $sysWeb->addChild('handlers');
		
		$addWeb = $handler->addChild('add');	
		$addWeb->addAttribute('name', "PHP via FastCGI");
		$addWeb->addAttribute('path', "*.php");
		$addWeb->addAttribute('verb', "*");
		$addWeb->addAttribute('modules', "FastCgiModule");
		$addWeb->addAttribute('scriptProcessor', "%RoleRoot%\approot\php\php-cgi.exe");
		$addWeb->addAttribute('resourceType', "Unspecified");				
			
		$docDefault=$sysWeb->addChild('defaultDocument')->addChild('files')->addChild('add');
		$fileValue=$this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEFAULT_DOC];
		$docDefault->addAttribute('value', $fileValue);	
		
		return $xml;
		
	}
	
	
	
	/**
   	 * Add DLLs  for HWC
   	 * 
   	 * @return void
   	 */
	private function _addHwcDlls()
	{
	
		
			
		if(!is_null($this->_oToolContext->strHostedWebCorePath))
			$this->_addHwcSource();
		
		
		copy(
        	"{$this->_oToolContext->strResToolDir}/".WACmdLineTools_Util_Constants::WORKER_ROLE_HWCSERVER_DLL,
            "{$this->_oToolContext->strWorkerRoleDir}/".WACmdLineTools_Util_Constants::WORKER_ROLE_HWCSERVER_DLL
        );   
		
		
		
		copy(
        	"{$this->_oToolContext->strResToolDir}/applicationHost.config",
            "{$this->_oToolContext->strWorkerRoleDir}/applicationHost.config"
        ); 

		copy(
        	"{$this->_oToolContext->strResToolDir}/webworker.config",
            "{$this->_oToolContext->strWorkerRoleDir}/web.config"
        ); 
		
		
		
		return true;
	}
	/**
   	 * Add source  for HWC
   	 * 
   	 * @return void
   	 */
	private function _addHwcSource()
	{
		
		try {  
									
			WACmdLineTools_Util::copyDir(
				$this->_oToolContext->strHostedWebCorePath, 
				$this->_oToolContext->strWorkerRoleDir."\\".WACmdLineTools_Util_Constants::WORKER_ROLE_WEBSITE_DIR,
				false
			);
						
			copy(
        		"{$this->_oToolContext->strResToolDir}/".WACmdLineTools_Util_Constants::WORKER_ROLE_WEB_CONFIG,
            	"{$this->_oToolContext->strWorkerRoleDir}\\".WACmdLineTools_Util_Constants::WORKER_ROLE_WEBSITE_DIR."\\web.config"
        	); 
		    $this->_addWindowsAzureSdk4PHP($this->_oToolContext->strWorkerRoleDir."\\".WACmdLineTools_Util_Constants::WORKER_ROLE_WEBSITE_DIR);
		
		}catch(Exception $e){
		
			throw new RuntimeException( 
				"Failed to add HWC php source directory" 
			);
		}
		return true;
			
	}
}
