<?php
/**
 * ServiceConfig.php
 * 
 * Setup Service Configuration for deployment
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

/**
 * Configure XML files for Deployment
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Stages_ServiceConfig
{
	private $_oToolContext 			= null;
	private $_strDefaultDoc			= null;	
	private $_iAppInstances			= 1;

    const ENUM_WEB_ROLE_SET_SERVICE_CONFIG      = "web_role_service_config";
    const ENUM_WEB_ROLE_SET_SERVICE_DEF       	= "web_role_service_def";
    const ENUM_WEB_CONFIG_SET_DEFAULT_DOC 		= "web_config_set_default_doc";
    const ENUM_WEB_CONFIG_VALIDATE_DEFAULT_DOC	= "web_config_validate_default_doc";
		   	
    /**
     * A private constructor; prevents direct 
     * creation of this object
     * 
     * @return void
     */
    public function __construct() 
    {
    	$this->_init();
    }

   	
    /**
     * Initialize singleton instance.
     * 
     * @return void
     */
    private function _init()
    {
		$this->_oToolContext = WACmdLineTools_Stages_Context::getInstance();
		if ( is_null($this->_oToolContext) ) {
			throw new RuntimeException(
				"Failed to create instance of WACmdLineTools_Stages_Context" 
			);			
		}
		
		$this->_strDefaultDoc
			= $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEFAULT_DOC];

		$this->_iAppInstances
			= $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_WEB_ROLE_INSTANCES];
    }
   	  	
	/**
	 * Process endpoint to this class
	 * 
	 * @param enum $eConfigRequest Configure based upon staging
	 * 
	 * @return boolean Returns TRUE upon success, else FALSE
	 */
	public function run($eConfigRequest) 
	{
        switch ($eConfigRequest) {
        case self::ENUM_WEB_ROLE_SET_SERVICE_CONFIG:
			$this->_oToolContext->verbose("Configuring \"{$this->_oToolContext->strProjectName}\" .");
						
			$this->_configServiceDefinitionFile();
			$this->_configServiceConfigurationFile();
			break;
			
        case self::ENUM_WEB_CONFIG_SET_DEFAULT_DOC:
			if ( strcmp($this->_strDefaultDoc, WACmdLineTools_Util_Constants::FILE_DEFAULT_DOC) != 0 ) {
				$this->_oToolContext->verbose("Set Web.config's Default Document.");
				$this->_setWebConfigDefaultDocument();
				$this->_validateWebConfigDefaultDoc();
			}
        	break;
        				
        case self::ENUM_WEB_CONFIG_VALIDATE_DEFAULT_DOC:
			$this->_oToolContext->verbose("Validating Web.config's Default Document exists.");
        	$this->_validateWebConfigDefaultDoc();
        	break;
        	
        case self::ENUM_WEB_ROLE_SET_SERVICE_DEF:
			$this->_oToolContext->verbose("Configuring \"{$this->_oToolContext->strProjectName}\" Service.");
			
			$this->_configServiceDefinitionRd();			
            break;
            
        default:
            throw new LogicException( "Unexpected configuration request: \"{$eConfigRequest}\"");
            break;
        }
				
		return true;	
	}
	   	
   	/**
   	 * Get default document within Web.config file.
   	 * 
   	 * @param string &$sDefaultDocument OUT Default Document
   	 * 
	 * @return boolean Returns TRUE upon success, else FALSE
   	 */
   	private function _getWebConfigDefaultDocument(&$sDefaultDocument)
   	{
   		$sDefaultDocument = null;
   		$bStatus = false;

		$strDefaultDocPath = "{$this->_oToolContext->strWebRoleDir}/{$this->_strDefaultDoc}";
   		
		if ( !is_file($strDefaultDocPath) ) {
   			throw new RuntimeException(
   				"Unable to find default document"
   				. " \"{$this->_strDefaultDoc}\""
   				. " does not exist in"
   				. " WebRole directory"
	   			. " \"{$this->_oToolContext->strWebRoleDir}\""
   			);
   		}

   		$strWebConfigPath  = "{$this->_oToolContext->strWebRoleDir}/";
   		$strWebConfigPath .= WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG;
   		
   		if ( !is_file($strWebConfigPath) ) {
   			throw new RuntimeException("File \"" 
   				. WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG 
   				. "\" does not exist in project \"{$this->_oToolContext->strProjectName}\"" 				
   				);
   		} 		
		
		try
		{			
			$xml = simplexml_load_file($strWebConfigPath);
			try {
					$xmlDefaultDocument = $xml->xpath("system.webServer/defaultDocument/files/add");					
            } catch(Exception $e) 
				{
					$xmlDefaultDocument = $xml->xpath("system.webServer/defaultDocument/files/clear/add");
            }
				
            if (count($xmlDefaultDocument)==0) {
				  $xmlDefaultDocument = $xml->xpath("system.webServer/defaultDocument/files/clear/add");
			}
			
			$sDefaultDocument = $xmlDefaultDocument[0]["value"];
			
			$bStatus = true;	
		}
		catch(Exception $ex)
		{
			throw new RuntimeException ("Failed to load XML File \""
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG
				. "\""
				
				);
		}
   		
   		return $bStatus;	
   	}
   	
   	/**
   	 * Declare default document within Web.config file.
   	 * 
   	 * @return boolean Returns TRUE upon success, else FALSE.
   	 */
   	private function _setWebConfigDefaultDocument()
   	{
   		$bStatus = false;
		$this->_oToolContext->verbose(
			"Set \"{$this->_strDefaultDoc}\""
			. " as the default document for project"
			. " \"{$this->_oToolContext->strProjectName}\".",
			false
		);

		$strDefaultDocPath = "{$this->_oToolContext->strWebRoleDir}/{$this->_strDefaultDoc}";
   		
		if ( !is_file($strDefaultDocPath) ) {
   			throw new RuntimeException(
   				"Unable to set default document"
   				. " \"{$this->_strDefaultDoc}\""
   				. " does not exist in"
   				. " WebRole directory"
	   			. " \"{$this->_oToolContext->strWebRoleDir}\""
   			);
   		}

   		$strWebConfigPath  = "{$this->_oToolContext->strWebRoleDir}/";
   		$strWebConfigPath .= WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG;
   		
   		if ( !is_file($strWebConfigPath) ) {
   			throw new RuntimeException("File \"" 
   				. WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG 
   				. "\" does not exist in project \"{$this->_oToolContext->strProjectName}\""
   				);
   		} 		
		
		try
		{			

			$xml = simplexml_load_file($strWebConfigPath);
			
			try {
					$xmlDefaultDocument = $xml->xpath("system.webServer/defaultDocument/files/add");					
					
			} catch(Exception $e) 
				{
					$xmlDefaultDocument = $xml->xpath("system.webServer/defaultDocument/files/clear/add");
					
			}
				
			if (count($xmlDefaultDocument)==0) {
				  $xmlDefaultDocument = $xml->xpath("system.webServer/defaultDocument/files/clear/add");
			}

			unset($xmlDefaultDocument[0]["value"]);
			$xmlDefaultDocument[0]->addAttribute("value", $this->_strDefaultDoc);
			
			file_put_contents($strWebConfigPath, $xml->asXML()); 
			
			$bStatus = true;	
		}
		catch(Exception $ex)
		{
			throw new RuntimeException ("Failed to load XML File \""
				. WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG
				. "\""
				);
		}
   		
   		return $bStatus;	
   	}
   	
   	/**
   	 * Validate that Web.config's Default Document value is
   	 * a file that does exist within the WebRole folder
   	 * 
   	 * @return void
   	 */
   	private function _validateWebConfigDefaultDoc()
   	{
		$this->_getWebConfigDefaultDocument($strDefaultDoc);
		$strDefaultDocPath = "{$this->_oToolContext->strWebRoleDir}/{$strDefaultDoc}";
		if ( !is_file($strDefaultDocPath) ) {
			throw new WACmdLineTools_Stages_Resources_Exception (
				"Web.config's Default Document value \"{$strDefaultDoc}\" is not in WebRole",
				WACmdLineTools_Stages_Resources_Exception::RES_NOT_AVAILABLE
			);
		}
   	}
   	
   	/**
   	 * Set service name within Service Definition file.
   	 * 
   	 * @return boolean Returns TRUE upon success, else FALSE.
   	 */   	
   	private function _configServiceDefinitionFile()
   	{
   		$bStatus = false;
		$this->_oToolContext->verbose(
			"Set \"" 
			. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF 
			. "\""
			. " with serviceName of"
			. " \"{$this->_oToolContext->strProjectName}\".",
			false
		);
		
		$strServiceDefPath  
			= "{$this->_oToolContext->strServiceDir}/"
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF;
   		
		if ( !is_file($strServiceDefPath) ) {
   			throw new RuntimeException(
   				"Default document"
   				. " \"" 
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF 
				. "\""
   				. " does not exist in"
   				. " Service directory"
	   			. " \"{$this->_oToolContext->strServiceDir}\""
   			);
   		}
   		
		try
		{			
			$xml = simplexml_load_file($strServiceDefPath);
			
			unset($xml[0]["name"]);
			$xml[0]->addAttribute("name", $this->_oToolContext->strProjectName);
			
			file_put_contents($strServiceDefPath, $xml->asXML());
			
			$bStatus = true;	
		}
		catch(Exception $ex)
		{
			throw new RuntimeException ("Failed to configure \""
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF
				. "\""				
				);
		}
   		
   		return $bStatus;
   	}
   	
   	/**
   	 * Set service name within Service Cofigur file.
   	 * 
   	 * @return boolean Returns TRUE upon success, else FALSE.
   	 */   	
   	private function _configServiceConfigurationFile()
   	{
   		$bStatus = false;
		$this->_oToolContext->verbose(
			"Set \"" 
			. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG 
			. "\""
			. " with serviceName of"
			. " \"{$this->_oToolContext->strProjectName}\".",
			false
		);
		
		$strServiceConfigPath  = "{$this->_oToolContext->strServiceDir}/";
		$strServiceConfigPath .= WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG;
   		
		if ( !is_file($strServiceConfigPath) ) {
   			throw new RuntimeException(
   				"Default document"
   				. " \"" 
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG 
				. "\""
   				. " does not exist in"
   				. " Service directory"
	   			. " \"{$this->_oToolContext->strServiceDir}\""  				
   			);
   		}
   		
		try
		{		
			$xml = simplexml_load_file($strServiceConfigPath);	
			
			unset($xml[0]["serviceName"]);
			$xml[0]->addAttribute("serviceName", $this->_oToolContext->strProjectName);
	
			if ( $this->_iAppInstances > 1 ) {
				$xmlDefaultDocument = $xml->Role->Instances;
				unset($xmlDefaultDocument[0]["count"]);
				$xmlDefaultDocument[0]->addAttribute("count", $this->_iAppInstances);
			}					
			
			file_put_contents($strServiceConfigPath, $xml->asXML());
			
			$bStatus = true;	
		}
		catch(Exception $ex)
		{
			throw new RuntimeException ("Failed to configure \""
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG
				. "\""				
				);
		}
   		
   		return $bStatus;
   	}

   	/**
   	 * Set CGI handler within Service Definition RD file.
   	 * 
   	 * @return boolean Returns TRUE upon success, else FALSE.
   	 */   	
   	private function _configServiceDefinitionRd()
   	{
   		$bStatus = false;
		$this->_oToolContext->verbose(
			"Configuring \"" 
			. WACmdLineTools_Util_Constants::FILE_SERVICE_DEF_RD 
			. "\""
			. " with FastCGI handling of WebRole.",
			false
		);

		$strServiceDefRdPath  = "{$this->_oToolContext->strServiceDir}/";
		$strServiceDefRdPath .= WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_CSX;
		$strServiceDefRdPath .= "/";
		$strServiceDefRdPath .= WACmdLineTools_Util_Constants::FILE_SERVICE_DEF_RD;

		if ( !is_file($strServiceDefRdPath) ) {
   			throw new RuntimeException(
   				"Default document"
   				. " \"" 
				. WACmdLineTools_Util_Constants::FILE_SERVICE_DEF_RD 
				. "\""
   				. " does not exist in"
   				. " Service directory"
	   			. " \""
	   			. WACmdLineTools_Util_Constants::DIR_SERVICE_DEF_CSX
	   			. "\""
   			);
   		}
   		
		try
		{			
			$xml = simplexml_load_file($strServiceDefRdPath);

			$xmlDefaultDocument = $xml->groups->group->components->groupHascomponents->role;

			unset($xmlDefaultDocument[WACmdLineTools_Util_Constants::AZURE_WEB_ROLE_HANDLER_ATTRIB]);

			$xmlDefaultDocument->addAttribute(
				WACmdLineTools_Util_Constants::AZURE_WEB_ROLE_HANDLER_ATTRIB, 
				WACmdLineTools_Util_Constants::AZURE_WEB_ROLE_HANDLER_VALUE
			);
			
			file_put_contents($strServiceDefRdPath, $xml->asXML()); 
			
			$bStatus = true;	
		}
		
		catch(Exception $ex)
		{
			throw new RuntimeException ("Failed to configure \""
				. WACmdLineTools_Util_Constants::FILE_SERVICE_DEF_RD
				. "\""				
				);
		}
   		
   		return $bStatus;
   	}
}