<?php
/**
 * ValidateSetup.php
 * 
 * Validate setup of local host for performing Deployment
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */


/**
 * Validate setup of local host for performing deployment class
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Stages_ValidateSetup
{
	private $_oToolContext 			= null;
	private $_bStop					= false;

	const ENUM_VALIDATE_TOOL_RES 					= "validate_tool_res";
			
	const ENUM_VALIDATE_PHP_SOURCE 					= "validate_php_source";
	const ENUM_VALIDATE_DEFAULT_DOC 				= "validate_default_doc";

    const ENUM_VALIDATE_WEB_ROLE 					= "validate_web_role";
    const ENUM_VALIDATE_SERVICE  					= "validate_service";

	const ENUM_VALIDATE_WORKER_ROLE  				= "validate_worker_role";
	const ENUM_VALIDATE_WORKER_ROLE_SCRIPT_PATH 	= "validate_worker_role_script_path";
	const ENUM_VALIDATE_SCHEDULE_TIME_DURATION 		= "validate_script_duration";
	const ENUM_VALIDATE_HOSTED_WEB_CORE_PATH 		= "validate_hosted_web_core_path";
	   	
    /**
     * A private constructor; prevents direct 
     * creation of this object
     * 
     * @return void
     */
    public function __construct() 
    {
    	$this->_init();
    }
    
    /**
     * Initialize singleton instance.
     * 
     * @return void
     */
    private function _init()
    {
		$this->_oToolContext = WACmdLineTools_Stages_Context::getInstance();
		if ( is_null($this->_oToolContext) ) {
			throw new RuntimeException(
				"Failed to create instance of WACmdLineTools_Stages_Context" 
			);			
		}   	
    }
   	
	/**
	 * Process endpoint to this class
	 * 
     * @param enum $eSetupValidRequest Package based upon staging
	 * 
	 * @return boolean True if success, else False
	 */
   	public function run($eSetupValidRequest)
	{
        switch ($eSetupValidRequest) {
        	
		case self::ENUM_VALIDATE_TOOL_RES:
			$this->_validateToolPhpRes();
			break;
        	       	
        case self::ENUM_VALIDATE_PHP_SOURCE: 
			$this->_validatePhpSource();
        	break;
			
        case self::ENUM_VALIDATE_DEFAULT_DOC: 
			$this->_validateDefaultDoc();
        	break;
			
        case self::ENUM_VALIDATE_WEB_ROLE:
		
            // code added for not displaying if web role is not present
			if ( !is_null($this->_oToolContext->strSourcePath) ) {
			
				$this->_oToolContext->verbose("Validating WebRole Directory.");
			}

			if ( !$this->_validateTargetDir() ) {
				return;
			} 
			
			if ( !$this->_validateBuildDir() ) {
				return;
			} 
			
			if ( !$this->_validateProjectWebRoleDir() ) {
				return;
			}
			
			if ( !$this->_validateWebRolePathLength() ) {
				return;
			}

			if ( !$this->_validateProjectServiceDir() ) {
				return;
			}

            break;

        case self::ENUM_VALIDATE_SERVICE:
            $this->_oToolContext->verbose("Validating Service Directory.");
        	        	
            break;
			
		case self::ENUM_VALIDATE_WORKER_ROLE:
            $this->_validateWorkerRoleSource();	
            break;
		
		case self::ENUM_VALIDATE_WORKER_ROLE_SCRIPT_PATH:
            $this->_validateWorkerRoleScriptPath();	
            break;
		
		case self::ENUM_VALIDATE_SCHEDULE_TIME_DURATION:
			$this->_validateScriptTimeDuration();	
            break;
		
		case self::ENUM_VALIDATE_HOSTED_WEB_CORE_PATH:
            $this->_validateHostedWebCorePath();	
            break;

					
        default:
            throw new LogicException( "Unexpected validate setup request: \"{$eSetupValidRequest}\"");
            break;
        }		
		return true;	
	}

	/**
	 * Validate PHP sources to by provide to WebRole directory.
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */
	private function _validateToolPhpRes()
	{
		$this->_oToolContext->verbose("Validating Tool's PHP Resource.");
		$bIsValid = false;
		
		if ($this->_oToolContext->bCopyPhpRuntimeDir) {

			if ( is_null($this->_oToolContext->strPhpRuntimeDir)
				|| !is_dir($this->_oToolContext->strPhpRuntimeDir)
			) {
				throw new WACmdLineTools_Stages_Resources_Exception(
					"--"
					. WACmdLineTools_Stages_Params::OPT_L_PHP_RESOURCES_DIR
					. " (-"
					. WACmdLineTools_Stages_Params::OPT_S_PHP_RESOURCES_DIR
					. ") "
					. "needs to provide a valid path to PHP runtime",
					WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
				);
			}
			
			if (WACmdLineTools_Util::getDirFileCount($this->_oToolContext->strPhpRuntimeDir) == 0) {
				throw new WACmdLineTools_Stages_Resources_Exception(
					"--"
					. WACmdLineTools_Stages_Params::OPT_L_PHP_RESOURCES_DIR
					. " (-"
					. WACmdLineTools_Stages_Params::OPT_S_PHP_RESOURCES_DIR
					. ") "
					. "is empty",
					WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
				);			
			}
			
			$this->_oToolContext->verbose(
				"Copying PHP Runtime "
				. "\"{$this->_oToolContext->strPhpRuntimeDir}\" "
				. "to Tool's PHP Resources.",
				false
			);
				
			if (is_dir($this->_oToolContext->strResPhpRuntimeDir)) {
				WACmdLineTools_Util::doRemoveDir($this->_oToolContext->strResPhpRuntimeDir);
			}
			
			WACmdLineTools_Util::doCopyDir(
				$this->_oToolContext->strPhpRuntimeDir, 
				$this->_oToolContext->strResPhpRuntimeDir,
				true
			);
		}
		
		if ( !is_dir($this->_oToolContext->strResPhpRuntimeDir) 
			|| WACmdLineTools_Util::getDirFileCount($this->_oToolContext->strResPhpRuntimeDir) == 0
		) {
			throw new WACmdLineTools_Stages_Resources_Exception(
				"User has not provided PHP Runtime. Provide path to PHP Runtime using parameter -r.",
				WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
			);
		}
		
   		$strZipFile = WACmdLineTools_Util_Constants::FILE_PHP_AZURE_SDK_ZIP;
   		$strZipFilePath = "{$this->_oToolContext->strResPhpDir}/{$strZipFile}";
		if (!is_file($strZipFilePath)) {
   			throw new WACmdLineTools_Stages_Resources_Exception ( 
	   			"ZIP file \"{$strZipFile}\" does not exist within Tool's \\res\\php\\ directory."
   				. " Download \"Windows Azure SDK for PHP\" and add it",
   				WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
   			);
   		}		
		return true;
	}
	
	/**
	 * Validate path length of WebRole directory does not exceed CSPack expectations.
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */
	private function _validateWebRolePathLength() 
	{
		$this->_oToolContext->verbose("Validating WebRole path length", false);
		
		$iWebRolePathLength = strlen($this->_oToolContext->strWebRoleDir);
		if ( $iWebRolePathLength >= 248 ) {
			throw new WACmdLineTools_Stages_Resources (
				"WebRole path length {$iWebRolePathLength} is too long after being fully qualified. Make sure the full path is less than 260 characters and the directory name is less than 248 characters."
			);
		}

		return true;
	}
	
	/**
	 * Validate PHP sources to by provide to WebRole directory.
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */
	private function _validatePhpSource()
	{
		$this->_oToolContext->verbose("Validating PHP Sources for WebRole.", false);
		
		$bPhpSourcesProvided = true;
		if ( is_null($this->_oToolContext->strSourcePath) 
			|| !is_string($this->_oToolContext->strSourcePath)
			|| empty($this->_oToolContext->strSourcePath) 
		) {
			//$bPhpSourcesProvided = false;
			$bPhpSourcesProvided = true; 
		}
		
		if ( !$bPhpSourcesProvided ) {
			$this->_oToolContext->bAddDefaultPhpSource = true;
			$this->_oToolContext->strSourcePath 
				= "{$this->_oToolContext->strResToolDir}/"
				. WACmdLineTools_Util_Constants::RES_FILE_INDEX_PHP;
		}
		
		if ( is_dir($this->_oToolContext->strSourcePath)
			&& ( strcasecmp($this->_oToolContext->strTargetDir, $this->_oToolContext->strSourcePath) == 0 ) 
		) {
			throw new RuntimeException(
				"Conflict: directories referenced by parameters \""
				. WACmdLineTools_Stages_Params::OPT_L_SOURCE_PATH
				. "\" and \""
				. WACmdLineTools_Stages_Params::OPT_L_TARGET_DIR
				. "\" cannot be the same" 
			);
		}
	
		if ( is_dir($this->_oToolContext->strSourcePath) 
			&& ( WACmdLineTools_Util::getDirFileCount($this->_oToolContext->strSourcePath) == 0 )
		) {			
			throw new RuntimeException( 
				"Source directory"
				. " \"{$this->_oToolContext->strSourcePath}\" is empty" 
			);
		}

		if ( is_dir($this->_oToolContext->strSourcePath)
			&& WACmdLineTools_Util::inDir($this->_oToolContext->strWorkingDir, $this->_oToolContext->strSourcePath)
		) {
			throw new RuntimeException(
				"Conflict: \""
				. WACmdLineTools_Stages_Params::OPT_L_SOURCE_PATH
				. "\" cannot be within"
				. " Toolkit directory"
			);			
		}
			
		if ( WACmdLineTools_Util::inDir($this->_oToolContext->strWorkingDir, $this->_oToolContext->strTargetDir) ) {
			throw new RuntimeException(
				"Conflict: \""
				. WACmdLineTools_Stages_Params::OPT_L_TARGET_DIR
				. "\" cannot be within"
				. " Toolkit directory"
			);			
		}

		if ( is_dir($this->_oToolContext->strSourcePath) ) {			
			$strDir = "{$this->_oToolContext->strSourcePath}/" . WACmdLineTools_Util_Constants::DIR_RESERVED_BIN;
			if ( is_dir($strDir)) {
				throw new RuntimeException( 
					"Conflict: \""
					. WACmdLineTools_Stages_Params::OPT_L_SOURCE_PATH
					. "\" has a reserved directory labeled \""  . WACmdLineTools_Util_Constants::DIR_RESERVED_BIN . "\""
				);
			}	
			
			$strDir = "{$this->_oToolContext->strSourcePath}/" . WACmdLineTools_Util_Constants::DIR_RESERVED_MICROSOFT;
			if ( is_dir($strDir)) {
				throw new RuntimeException( 
					"Conflict: \""
					. WACmdLineTools_Stages_Params::OPT_L_SOURCE_PATH
					. "\" has a reserved directory labeled \""  . WACmdLineTools_Util_Constants::DIR_RESERVED_MICROSOFT . "\""
				);
			}		
		}
		
		return true;
	}
	
	/**
	 * Validate Default document provided WebRole directory.
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */
	private function _validateDefaultDoc()
	{
		$this->_oToolContext->verbose("Validating default document within project source for WebRole.", false);
		
		$bPhpSourcesProvided = true;
		if ( is_null($this->_oToolContext->strSourcePath) 
			|| !is_string($this->_oToolContext->strSourcePath)
			|| empty($this->_oToolContext->strSourcePath) 
		) {
			$bPhpSourcesProvided = false;
		}
		
		if ( !$bPhpSourcesProvided ) {
			return true;
		}
		
		$this->_getPhpSourceDefaultDoc($sDefaultDoc);
		
		if ( $this->_oToolContext->bAddDefaultPhpSource ) {
			$sDefaultDocPath = "{$this->_oToolContext->strResToolDir}/{$sDefaultDoc}";
		} else {
			$sDefaultDocPath = "{$this->_oToolContext->strSourcePath}/{$sDefaultDoc}";
		}
		
		if ( !is_file($sDefaultDocPath) ) {
			throw new WACmdLineTools_Stages_Resources_Exception (
				"Failed to find project default document \"{$sDefaultDocPath}\""
				);			
		}
		
		return true;
	}

	/**
	 * Validate Target Directory
	 * 
	 * @return boolean Returns TRUE if found, else FALSE 
	 */
	private function _validateTargetDir()
	{
		$bResult = $this->_validateDir("Target", $this->_oToolContext->strTargetDir);
			
		if ( !$bResult && $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_PACKAGE] ) {
			throw new WACmdLineTools_Stages_Params_Exception(
				"--"
				. WACmdLineTools_Stages_Params::OPT_L_PACKAGE
				. " (-"
				. WACmdLineTools_Stages_Params::OPT_L_PACKAGE
				. ") "
				. "Missing Target directory, Need to perform build before packaging"
			);				
		}
								
		if ( !$bResult && $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEPLOY] ) {
			throw new WACmdLineTools_Stages_Params_Exception(
				"--"
				. WACmdLineTools_Stages_Params::OPT_L_DEPLOY
				. " (-"
				. WACmdLineTools_Stages_Params::OPT_S_DEPLOY
				. ") "
				. "Missing Target directory, Need to perform build and packaging before deployment"
			);				
		}
		
		return $bResult;
	}
	
	/**
	 * Validate Build Directory
	 * 
	 * @return boolean Returns TRUE on success, else FALSE 
	 */
	private function _validateBuildDir()
	{
		$bResult = $this->_validateDir("Build", $this->_oToolContext->strBuildDir);
			
		if ( !$bResult && $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_PACKAGE] ) {
			throw new WACmdLineTools_Stages_Params_Exception(
				"--"
				. WACmdLineTools_Stages_Params::OPT_L_PACKAGE
				. " (-"
				. WACmdLineTools_Stages_Params::OPT_L_PACKAGE
				. ") "
				. "Missing Build directory, Need to perform build before packaging"
			);				
		}

		if ( !$bResult && $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEPLOY] ) {
			throw new WACmdLineTools_Stages_Params_Exception(
				"--"
				. WACmdLineTools_Stages_Params::OPT_L_DEPLOY
				. " (-"
				. WACmdLineTools_Stages_Params::OPT_S_DEPLOY
				. ") "
				. "Missing Build directory, Need to perform build and packaging before deployment"
			);				
		}
		
		return $bResult;		
	}
		
	/**
	 * Validate WebRole directory
	 * 
	 * @return boolean Returns TRUE on success, else FALSE 
	 */	
	private function _validateProjectWebRoleDir()
	{
		$bResult = $this->_validateDir("WebRole", $this->_oToolContext->strWebRoleDir);
		
		if ( !$bResult && $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_PACKAGE] ) {
			throw new WACmdLineTools_Stages_Params_Exception(
				"--"
				. WACmdLineTools_Stages_Params::OPT_L_PACKAGE
				. " (-"
				. WACmdLineTools_Stages_Params::OPT_L_PACKAGE
				. ") "
				. "Missing WebRole directory, Need to perform build before packaging"
			);				
		}
								
		if ( !$bResult && $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEPLOY] ) {
			throw new WACmdLineTools_Stages_Params_Exception(
				"--"
				. WACmdLineTools_Stages_Params::OPT_L_DEPLOY
				. " (-"
				. WACmdLineTools_Stages_Params::OPT_S_DEPLOY
				. ") "
				. "Missing WebRole directory, Need to perform build and packaging before deployment"
			);				
		}
		
		return $bResult;	
	}
	
	/**
	 * Validate Service directory
	 * 
	 * @return boolean Returns TRUE on success, else FALSE 
	 */		
	private function _validateProjectServiceDir()
	{
		$bResult = $this->_validateDir("Service", $this->_oToolContext->strServiceDir);	
		
		if ( !$bResult && $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEPLOY] ) {
			throw new WACmdLineTools_Stages_Params_Exception(
				"--"
				. WACmdLineTools_Stages_Params::OPT_L_DEPLOY
				. " (-"
				. WACmdLineTools_Stages_Params::OPT_S_DEPLOY
				. ") "
				. "Missing Service directory, Need to perform build and packaging before deployment"
			);				
		}	
		
		if (!$bResult) {
			return true;
		}
		
		if ( !is_file($this->_oToolContext->strServiceDefFile) ) {
			$this->_oToolContext->verbose("New build. Service Definition file does not exist.", false);
			
			if ( $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEPLOY] ) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. WACmdLineTools_Stages_Params::OPT_L_DEPLOY
					. " (-"
					. WACmdLineTools_Stages_Params::OPT_S_DEPLOY
					. ") "
					. "Build is required"
				);				
			}
			if ( $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_PACKAGE] ) {
				throw new WACmdLineTools_Stages_Params_Exception(
					"--"
					. WACmdLineTools_Stages_Params::OPT_L_DEPLOY
					. " (-"
					. WACmdLineTools_Stages_Params::OPT_S_DEPLOY
					. ") "
					. "Build is required"
				);				
			}			
			$this->_oToolContext->bDoCleanRebuild = true;
			
			return false;
		}
		
		$iServiceDefFile = strlen($this->_oToolContext->strServiceDefFile);

		return true;
	}
	
	/**
	 * Add user provided Source to project's WebRole directory.
	 * 
	 * @param boolean &$bIsValid Was directory determined as valid?
	 * 
	 * @return boolean Returns TRUE on success, else FALSE 
	 */	
	private function _validateSourceInWebRole(&$bIsValid)
	{
		$bIsValid = false;
		
		$this->_oToolContext->verbose("Validate PHP source into \"{$this->_oToolContext->strProjectName}\" WebRole directory", false);
		
		$strSource = basename($this->_oToolContext->strSourcePath);
		$strWebRoleSource = "{$this->_oToolContext->strWebRoleDir}/{$strSource}";
		
		if ( is_dir($this->_oToolContext->strSourcePath) 
			&& !is_dir($strWebRoleSource)
		) {
			throw new RuntimeException( 
				"PHP source directory not within WebRole directory" 		
			);		
		} else if ( is_file($this->_oToolContext->strSourcePath) 
			&& !is_file($strWebRoleSource)
		) {
			throw new RuntimeException( 
				"PHP source file not within WebRole directory" 
			);		
		}
		
		$bIsValid = true;

		return true;	
	}

	/**
	 * Support function to determine if directories for 
	 * setup are valid.
	 * 
	 * @param string $strDirName Short Directory name
	 * @param string $strDirPath Directory Path
	 * 
	 * @return boolean Returns TRUE if valid, else FALSE
	 */
	private function _validateDir($strDirName, $strDirPath)
	{
		$this->_oToolContext->verbose("Validating {$strDirName} directory exists", false); 
		
		if ( is_null($strDirPath)
			|| !is_string($strDirPath)
			|| empty($strDirPath)
		) {
			throw new InvalidArgumentException( 
				"{$strDirName} directory is not defined" 
			);	
		}
			
		if ( !is_dir($strDirPath) 
			|| ( WACmdLineTools_Util::getDirFileCount($strDirPath) == 0 ) 
		) {
			$this->_oToolContext->verbose("New build. {$strDirName} directory does not exist.", false);
			
			$this->_oToolContext->bDoCleanRebuild = true;
			return false;
		}

		return true;
	}
	
   	/**
   	 * Get default document within PHP source.
   	 * 
   	 * @param string &$sDefaultDoc OUT Default Document
   	 * 
	 * @return boolean Returns TRUE upon success, else FALSE
   	 */
   	private function _getPhpSourceDefaultDoc(&$sDefaultDoc)
   	{
   		$sDefaultDoc = null;
   		$bStatus = false;

   		 /* $sWebConfigPath  = "{$this->_oToolContext->strSourcePath}/";
   		$sWebConfigPath .= WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG;
   		
   		if ( is_file($sWebConfigPath) ) {
   			$this->_oToolContext->verbose(
   				"Getting default document from " . WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG . " file.", 
   				false
   			);
				
			try
			{	
				
				$xml = simplexml_load_file($sWebConfigPath);
				$xmlDefaultDocument = $xml->xpath("system.webServer/defaultDocument/files/add");
				$sDefaultDoc = $xmlDefaultDocument[0]["value"];
				$bStatus = true;	
			}
			catch(Exception $ex)
			{
				throw new RuntimeException ("Failed to load XML File \""
					. WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG
					. "\""				
					);
			}
			
			if($sDefaultDoc!=$this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEFAULT_DOC])
				{
					throw new RuntimeException ("Default docuement is miss match either user -c paramter or dafault document in \""
					. WACmdLineTools_Util_Constants::RES_FILE_WEB_CONFIG
					. "\""				
					);				 
				}
					
		$sDefaultDoc = $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEFAULT_DOC];
   		} else {
		*/
   			$this->_oToolContext->verbose(
   				"Getting default document from PHP source.", 
   				false
   			);
   			
   			$sDefaultDoc = $this->_oToolContext->aParams[WACmdLineTools_Stages_Params::OPT_L_DEFAULT_DOC];
			$bStatus = true;	
   		//}
		
   		return $bStatus;	
   	}

	/**
	 * Validate Worker role sources to by provide to WebRole directory.
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */
	private function _validateWorkerRoleSource()
	{
		$this->_oToolContext->verbose("Validating Worker Role Source directory.");
		
		
		$bIsValid = false;
		
		if (is_null($this->_oToolContext->strWorkerRoleScriptPath)) {
			throw new WACmdLineTools_Stages_Resources_Exception(
				"User has not provided Worker role start up script. Provide path to worker role start up script using parameter -b.",
				WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
			);
		}
		
		if ($this->_oToolContext->strWorkerRoleSourceDir) {

			if ( is_null($this->_oToolContext->strWorkerRoleSourceDir)
				|| !is_dir($this->_oToolContext->strWorkerRoleSourceDir)
			) {
				throw new WACmdLineTools_Stages_Resources_Exception(
					"--"
					. WACmdLineTools_Stages_Params::OPT_L_WORKERROLE_RESOURCES_DIR
					. " (-"
					. WACmdLineTools_Stages_Params::OPT_S_WORKERROLE_RESOURCES_DIR
					. ") "
					. "needs to provide a valid path to Worker Role Directory",
					WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
				);
			}
			
			if (WACmdLineTools_Util::getDirFileCount($this->_oToolContext->strWorkerRoleSourceDir) == 0) {
				throw new WACmdLineTools_Stages_Resources_Exception(
					"--"
					. WACmdLineTools_Stages_Params::OPT_L_WORKERROLE_RESOURCES_DIR
					. " (-"
					. WACmdLineTools_Stages_Params::OPT_S_WORKERROLE_RESOURCES_DIR
					. ") "
					. "is empty",
					WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
				);			
			}	
		}	
   		
		return true;
	}
	
	/**
	 * Validate Worker role startup sscript path.
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */
	private function _validateWorkerRoleScriptPath()
	{
		$this->_oToolContext->verbose("Validating Worker Role Start Up Script Path.");
				
		if (is_null($this->_oToolContext->strWorkerRoleSourceDir)) {
			throw new WACmdLineTools_Stages_Resources_Exception(
				"User has not provided Worker role source path. Provide worker role path using parameter -w.",
				WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
			);
		} else {
			$workerRoleStartupFile=$this->_oToolContext->strWorkerRoleSourceDir."/".$this->_oToolContext->strWorkerRoleScriptPath;
		
			if ($workerRoleStartupFile) {
			
				if ( is_null($workerRoleStartupFile)
					|| !is_file($workerRoleStartupFile)
				) {
					throw new WACmdLineTools_Stages_Resources_Exception(
						"--"
						. WACmdLineTools_Stages_Params::OPT_L_WORKERROLE_SCRIPT_PATH
						. " (-"
						. WACmdLineTools_Stages_Params::OPT_S_WORKERROLE_SCRIPT_PATH
						. ") "
						. "needs to provide a valid path to Worker Role Start Up File",
						WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
					);
				}
			}
		}   		
		return true;
	}	
	
	/**
	 * Validate Start up script time duration.
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */	 
	private function _validateScriptTimeDuration()
	{
		$this->_oToolContext->verbose("Validating Start Up Script Time Duration.");
				
		if ( is_null($this->_oToolContext->strWorkerRoleScriptPath) ) {
			throw new WACmdLineTools_Stages_Resources_Exception(
				"User has not provided Worker role start up script. Provide path to worker role start up script using parameter -b.",
				WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
			);
		} else {		
			$workerRoleStartupTimeDuration=$this->_oToolContext->strWorkerScriptTimeDuration;
			if ( is_null($workerRoleStartupTimeDuration) 
				|| $workerRoleStartupTimeDuration=="" 
			) {
				throw new WACmdLineTools_Stages_Resources_Exception(
						"--"
						. WACmdLineTools_Stages_Params::OPT_L_SCHEDULE_TIME_DURATION
						. " (-"
						. WACmdLineTools_Stages_Params::OPT_S_SCHEDULE_TIME_DURATION
						. ") "
						. "needs to provided a valid time duration for start up script (hh:mm:ss)",
						WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
				);
			} else if (strlen($workerRoleStartupTimeDuration) > 8) {
				throw new WACmdLineTools_Stages_Resources_Exception(
					"--"
					. WACmdLineTools_Stages_Params::OPT_L_SCHEDULE_TIME_DURATION
					. " (-"
					. WACmdLineTools_Stages_Params::OPT_S_SCHEDULE_TIME_DURATION
					. ") "
					. "needs to provided a valid time duration for start up script (hh:mm:ss)",
					WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
				);
			} else {
				$timeDurationArr=explode(":", $workerRoleStartupTimeDuration);
				$cntDuration=count($timeDurationArr);

				if ($cntDuration!=3 
					|| (trim($timeDurationArr[0])=="" || trim($timeDurationArr[1])=="" || trim($timeDurationArr[2])=="" )
					|| (strlen(trim($timeDurationArr[0]))>2 || strlen(trim($timeDurationArr[1]))>2 || strlen(trim($timeDurationArr[2]))>2  )
				) {
					throw new WACmdLineTools_Stages_Resources_Exception(
						"--"
						. WACmdLineTools_Stages_Params::OPT_L_SCHEDULE_TIME_DURATION
						. " (-"
						. WACmdLineTools_Stages_Params::OPT_S_SCHEDULE_TIME_DURATION
						. ") "
						. "needs to provided a valid time duration for start up script (hh:mm:ss)",
						WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
					);					
				} else if ( ereg('[^0-9]', trim($timeDurationArr[0])) 
					|| ereg('[^0-9]', trim($timeDurationArr[1])) 
					|| ereg('[^0-9]', trim($timeDurationArr[2]))
				) {
					throw new WACmdLineTools_Stages_Resources_Exception(
						"--"
						. WACmdLineTools_Stages_Params::OPT_L_SCHEDULE_TIME_DURATION
						. " (-"
						. WACmdLineTools_Stages_Params::OPT_S_SCHEDULE_TIME_DURATION
						. ") "
						. "needs to provided a valid time duration for start up script (hh:mm:ss)",
						WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
					);					
				}
			}
		}

		return true;
	}
	
	/**
	 * Validate Hosted Web Core path.
	 * 
	 * @return boolean Returns TRUE on success or FALSE on failure 
	 */
	private function _validateHostedWebCorePath()
	{
		$this->_oToolContext->verbose("Validating Hosted Web Core Directory Path.");
		
		if ($this->_oToolContext->strHostedWebCorePath) {
			if ( is_null($this->_oToolContext->strHostedWebCorePath)
				|| !is_dir($this->_oToolContext->strHostedWebCorePath)
			) {
				throw new WACmdLineTools_Stages_Resources_Exception(
					"--"
					. WACmdLineTools_Stages_Params::OPT_L_HOSTED_WEB_CORE_PATH
					. " (-"
					. WACmdLineTools_Stages_Params::OPT_S_HOSTED_WEB_CORE_PATH
					. ") "
					. "needs to provide a valid path to Hosted Web Core Directory",
					WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
				);
			}
					
			if (WACmdLineTools_Util::getDirFileCount($this->_oToolContext->strHostedWebCorePath) == 0) {
				throw new WACmdLineTools_Stages_Resources_Exception(
					"--"
					. WACmdLineTools_Stages_Params::OPT_L_HOSTED_WEB_CORE_PATH
					. " (-"
					. WACmdLineTools_Stages_Params::OPT_S_HOSTED_WEB_CORE_PATH
					. ") "
					. "is empty",
					WACmdLineTools_Stages_Resources_Exception::RES_REQUIRED
				);			
			}	
		}	
   		   		
		return true;
	}

		
}