<?php
/**
 * XDriveConfig.php
 * 
 * Upon request, mount an X-Drive
 * 
 * PHP Version 5
 * 
 * Copyright (c) 2010, Microsoft Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

/**
 * Mount an X-Drive class
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Stages_XDriveConfig
{
	private $_oToolContext		= null;
	
    const XDRIVE_CONFIG_NAME	= "XDrives";
    const XDRIVE_CONFIG_VALUE	= "[xdrive_label=MyDriveA,blob_container=mydrives,page_blob_vhd=MyDriveA.vhd,xdrive_sizeMB=40,mount_cache_sizeMB=20,create=Yes];";

    const XDRIVE_LOCAL_CACHE_NAME      				= "XDrivesLocalCache";
    const XDRIVE_LOCAL_CACHE_VALUE      			= "MyXDrivesLocalCache";

    
    const SERVICE_DEF_LOCAL_RESOURCES_TAG 			= "LocalResources";
	const SERVICE_DEF_LOCAL_STORAGE_TAG 			= "LocalStorage";
	
	const SERVICE_DEF_LOCAL_STORAGE_NAME			 = self::XDRIVE_LOCAL_CACHE_VALUE;
	
	const SERVICE_DEF_LOCAL_STORAGE_CLEAN_ATTR_NAME  = "cleanOnRoleRecycle";
	const SERVICE_DEF_LOCAL_STORAGE_CLEAN_ATTR_VALUE = "false";
	
	const SERVICE_DEF_LOCAL_STORAGE_SIZE_ATTR_NAME 	 = "sizeInMB";
    const SERVICE_DEF_LOCAL_STORAGE_SIZE_ATTR_VALUE  = "1000";
    
    /**
     * A private constructor; prevents direct 
     * creation of this object
     * 
     * @return void
     */
    public function __construct() 
    {
    	$this->_init();
    }

   	
    /**
     * Initialize singleton instance.
     * 
     * @return void
     */
    private function _init()
    {
		$this->_oToolContext = WACmdLineTools_Stages_Context::getInstance();
		if ( is_null($this->_oToolContext) ) {
			throw new RuntimeException(
				"Failed to create instance of WACmdLineTools_Stages_Context" 
			);			
		}
    }
    
	/**
	 * Process endpoint to this class
	 * 
	 * @return boolean Returns TRUE upon success, else FALSE
	 */
	public function run() 
	{
		if ( !$this->_configServiceConfigurationFile() ) {
			return false;
		}
		
		$this->_configServiceDefinitionFile();
		return true;	
	}
	
   	/**
   	 * Set service name within Service Definition file.
   	 * 
   	 * @return boolean Returns TRUE upon success, else FALSE.
   	 */   	
   	private function _configServiceDefinitionFile()
   	{
   		$bStatus = false;
		$this->_oToolContext->verbose(
			"Setup file \"" 
			. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF 
			. "\""
			. " with XDrive.",
			false
		);
		
		$strServiceDefPath  
			= "{$this->_oToolContext->strServiceDir}/"
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF;
   		
		if ( !is_file($strServiceDefPath) ) {
   			throw new RuntimeException(
   				"File "
   				. " \"" 
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF 
				. "\""
   				. " does not exist in"
   				. " Service directory"
	   			. " \"{$this->_oToolContext->strServiceDir}\""
   			);
   		}
   		
		try
		{			
			$xml = simplexml_load_file($strServiceDefPath);

			$xmlLocalResources = $xml->WebRole->LocalResources;
			
			$xmlLocalStorage = $xmlLocalResources->addChild(self::SERVICE_DEF_LOCAL_STORAGE_TAG);
			
			$xmlLocalStorage->addAttribute(
				"name", 
				self::SERVICE_DEF_LOCAL_STORAGE_NAME
			);
			
			$xmlLocalStorage->addAttribute(
				self::SERVICE_DEF_LOCAL_STORAGE_CLEAN_ATTR_NAME, 
				self::SERVICE_DEF_LOCAL_STORAGE_CLEAN_ATTR_VALUE
			);
			
			$xmlLocalStorage->addAttribute(
				self::SERVICE_DEF_LOCAL_STORAGE_SIZE_ATTR_NAME, 
				self::SERVICE_DEF_LOCAL_STORAGE_SIZE_ATTR_VALUE
			);
						
			file_put_contents($strServiceDefPath, $xml->asXML());
			
			$bStatus = true;	
		}
		catch(Exception $ex)
		{
			throw new RuntimeException ("Failed to configure \""
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_DEF
				. "\""
				. " with XDrive."			
				);
		}
   		
   		return $bStatus;
   	}
   	
   	/**
   	 * Set service name within Service Cofigur file.
   	 * 
   	 * @return boolean Returns TRUE upon success, else FALSE.
   	 */   	
   	private function _configServiceConfigurationFile()
   	{
   		$bStatus = false;
		$this->_oToolContext->verbose(
			"Setup File \"" 
			. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG 
			. "\""
			. " for one XDrive.",
			false
		);
		
		$strServiceConfigPath  = "{$this->_oToolContext->strServiceDir}/";
		$strServiceConfigPath .= WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG;
   		
		if ( !is_file($strServiceConfigPath) ) {
   			throw new RuntimeException(
   				"File "
   				. " \"" 
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG 
				. "\""
   				. " does not exist in"
   				. " Service directory"
	   			. " \"{$this->_oToolContext->strServiceDir}\""  				
   			);
   		}
   		
		try
		{		
			$xml = simplexml_load_file($strServiceConfigPath);
			
			$xmlSettings = $xml->Role->ConfigurationSettings->children();
			foreach ($xmlSettings as $xmlSetting) {
				if ( strcmp($xmlSetting["name"], self::XDRIVE_CONFIG_NAME) == 0 ) {
					$sValueCurrent = $xmlSetting[0]["value"];
					if ( !empty($sValueCurrent) && strlen($sValueCurrent) > 0 ) {
						return false;
					}
					
					unset($xmlSetting[0]["value"]);
					$xmlSetting[0]->addAttribute("value", self::XDRIVE_CONFIG_VALUE);
					continue;
				}
				
				if ( strcmp($xmlSetting["name"], self::XDRIVE_LOCAL_CACHE_NAME) == 0 ) {
					$sValueCurrent = $xmlSetting[0]["value"];
					if ( !empty($sValueCurrent) && strlen($sValueCurrent) > 0 ) {
						return false;
					}
					
					unset($xmlSetting[0]["value"]);
					$xmlSetting[0]->addAttribute("value", self::XDRIVE_LOCAL_CACHE_VALUE);
					continue;
				}
			}
						
			file_put_contents($strServiceConfigPath, $xml->asXML());
			
			$bStatus = true;	
		}
		catch(Exception $ex)
		{
			throw new RuntimeException ("Failed to configure \""
				. WACmdLineTools_Util_Constants::RES_FILE_SERVICE_CONFIG
				. "\""
				. " with XDrive."			
				);
		}
   		
   		return $bStatus;
   	}
}