<?php
/**
 * Util.php
 * 
 * Support functions
 * 
 * PHP Version 5
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

/**
 * Support class with basic functions.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
class WACmdLineTools_Util
{
	
    /**
     * Create directory request
     * 
     * @param string  $strDir         Path of directory to create
     * @param boolean $bAbortIfExists Query re-creating target directory if exists
     * @param octal   $mode           Changes access mode
     * @param boolean $bRecursive     Create parent directories if not exist.
     * 
     * @return boolean Returns TRUE on success or FALSE on failure
     */
    public static function createDir(
        $strDir,
        $bAbortIfExists = true,
        $mode = 0777,
        $bRecursive = true
    ) {
        if ( is_null($strDir) 
            || !is_string($strDir) 
            || empty($strDir)
        ) {
            throw new InvalidArgumentException ("Undefined \"strDir\"" );        
        }
                
        if ( is_dir($strDir) ) {
            if ( $bAbortIfExists ) {
                return false;
            }          
        }
        
        if ( is_dir($strDir) ) {
            chmod($strDir, $mode);
            if ( !self::doRemoveDir($strDir) ) {
                throw new RuntimeException( "Failed to delete \"{$strDir}\"." );
            }
            
            sleep(2);
        }
            
        if ( !mkdir($strDir, $mode, $bRecursive) 
            || !is_dir($strDir)
        ) {
            throw new RuntimeException( "Failed to create Build directory \"{$strDir}\"." );
        }

        return true;
    }
        
    /**
     * Fully copy a source directory to a target directory.
     * 
     * @param string  $strSourceDir   Source directory
     * @param string  $strTargetDir   Target directory
     * @param boolean $bAbortIfExists Query re-creating target directory if exists
     * @param octal   $mode           Changes access mode
     * 
     * @return boolean Returns TRUE on success or FALSE on failure
     */
    public static function copyDir(
        $strSourceDir, 
        $strTargetDir,
        $bAbortIfExists = true,
        $mode = 0777
    ) {
        if ( is_null($strSourceDir) 
            || !is_string($strSourceDir) 
            || empty($strSourceDir)
        ) {
            throw new InvalidArgumentException("Undefined \"strSourceDir\"");
        }
        
        if ( is_null($strTargetDir) 
            || !is_string($strTargetDir) 
            || empty($strTargetDir)
        ) {
        	throw new InvalidArgumentException("Undefined \"strTargetDir\"");
        }
                    
        if ( is_dir($strTargetDir) ) {
            if ( $bAbortIfExists ) {
                return false;
            }
        }
                        
        if ( is_dir($strSourceDir) ) {
            if ( !is_dir($strTargetDir) 
                && !mkdir($strTargetDir, $mode)
            ) {
                throw new RuntimeException ("Failed to make target directory \"{$strTargetDir}\"" );
            }
            $d = dir($strSourceDir);
            while ( false !== ( $entry = $d->read() ) ) {
                if ( $entry == '.' || $entry == '..' ) {
                    continue;
                }
                $strSourceEntry = $strSourceDir . '/' . $entry; 
                $strTargetEntry = $strTargetDir . '/' . $entry;
                if ( is_dir($strSourceEntry) ) {
                    self::copyDir(
                    	$strSourceEntry, 
                    	$strTargetEntry, 
                    	false, 
                    	$mode
                    );
                    continue;
                }
                if ( !copy($strSourceEntry, $strTargetEntry) ) {
                    throw new RuntimeException (
                        "Failed to copy"
                        . " file \"{$strSourceEntry}\""
                        . " to \"{$strTargetEntry}\"" 
                    );
                }
            }
            $d->close();
        } else {
            if ( !copy($strSourceDir, $strTargetDir) ) {
                throw new RuntimeException (
                    "Failed to copy"
                    . " file \"{$strSourceEntry}\""
                    . " to \"{$strTargetEntry}\"" 
                    
                );
            }
        }
        
        return true;
    }
    
    /**
     * Delete directory and all of its contents;
     * 
     * @param string $strDir Directory path
     * 
     * @return boolean Returns TRUE on success or FALSE on failure
     */
    public static function deleteDir($strDir) 
    {
        if ( is_null($strDir) 
            || !is_string($strDir) 
            || empty($strDir)
        ) {
            throw new InvalidArgumentException( "Undefined \"strDir\"" );        
        }
        
        $handleDir = false;
        if ( is_dir($strDir) ) {    
            $handleDir = @opendir($strDir);
        }
        if ( !$handleDir ) {
            return false;
        }
        chmod($strDir, 0777);
        while ( $file = readdir($handleDir) ) {
            if ( $file == '.' || $file == '..' ) {
                continue;
            }
            
            $fsEntity = $strDir."/".$file;
            
            if ( is_dir($fsEntity) ) {
                self::deleteDir($fsEntity);
                continue;
            }
            
            if (is_file($fsEntity)) {
                @unlink($fsEntity);
                continue;
            }
            
            throw new LogicException (
                "Unexpected file type: \"{$fsEntity}\"" 
            );
        }
        
        chmod($strDir, 0777);        
        closedir($handleDir);
        self::deleteDir($strDir);
                     
        return true;
    }
    
    
    /**
     * Determine if current operating system is Windows.
     * 
     * @return boolean
     */
    public static function isWindows()
    {
        return strtoupper(substr(PHP_OS, 0, 3)) === 'WIN';
    }
    
    /**
     * Transform path to local host expectations
     * 
     * @param string $strPath Path to transform
     * 
     * @return string Transformed path
     */
    public static function doPathTransform($strPath)
    {
        if ( WACmdLineTools_Util::isWindows() ) {
            $strPath = str_replace("/", "\\", $strPath);
        } else {
            $strPath = str_replace("\\", "/", $strPath);
        }

        return $strPath;
    }
    
    
    /**
     * Determine a File Extension 
     *  
     * @param string $strFileName File Name
     * 
     * @return string File Extension
     */
    public static function getFileExtention($strFileName)
    {
        return strtolower(substr(strrchr(basename($strFileName), '.'), 1));
    }

    /**
     * Get all file names within a directory.
     * 
     * @param string $strDirPath Directory path
     * 
     * @return array File names
     */
    public static function getDirFileNames($strDirPath)
    {
        if (!is_dir($strDirPath)) {
            throw new RuntimeException("\"{$strDirPath}\" is not a directory");
        }
        
        $aFilePaths = glob("" . rtrim($strDirPath, "\/\\ ") . "/*");
        
        $aFileNames = array();
        foreach ( $aFilePaths as $strFilePath ) {
        	$aFileNames[] = basename($strFilePath);
        }
        
        return $aFileNames;
    }
    
    /**
     * Count number of files within a directory.
     * 
     * @param string $strDirPath Directory path
     * 
     * @return int File count
     */
    public static function getDirFileCount($strDirPath)
    {
        return count(self::getDirFileNames($strDirPath));
    }
    
    /**
     * Determine if provided content (directory or file) is
     * within provide directory.
     * 
     * @param string $strDirPath   Root directory path
     * @param string $strCheckPath Dir or File path to check.
     * 
     * @return boolean True if found, else False
     */
    public static function inDir(
        $strDirPath, 
        $strCheckPath
    ) {    
        if ( is_null($strDirPath) 
            || !is_string($strDirPath) 
            || empty($strDirPath)
        ) {
            throw new InvalidArgumentException( 
                "Undefined \"strDirPath\"" 
                
            );
        }
        
        if ( is_null($strCheckPath) 
            || !is_string($strCheckPath) 
            || empty($strCheckPath)
        ) {
            throw new InvalidArgumentException( "Undefined \"strCheckPath\"" );
        }
        
        if ( strlen($strCheckPath) >= strlen($strDirPath) ) {    
            //$strDelimiter = "[\:\\\/]";
			$strDelimiter = "/[\s:\\\]/";			
			$aDirPath = preg_split($strDelimiter, $strDirPath);
            $aCheckPath = preg_split($strDelimiter, $strCheckPath);
            
            if ( count($aCheckPath) >= count($aDirPath)  ) {
                array_splice($aDirPath, count($aDirPath));
                array_splice($aCheckPath, count($aDirPath));
                
                return ($aDirPath === $aCheckPath);
            }
        }
        
        return false;                    
    }
    
    /**
     * Convert provided value into boolean
     * 
     * @param mix $mixVar Possible boolean value
     * 
     * @return boolean True if boolean value of true, else False
     */
    public static function boolVal($mixVar)
    {
        switch ($mixVar) {
        case $mixVar == true:
        case $mixVar == 1:
        case strtolower($mixVar) == 'true': 
        case strtolower($mixVar) == 'on':
        case strtolower($mixVar) == 'yes':
        case strtolower($mixVar) == 'y':
            $out = true;
            break;
        default: $out = false;
        }
        
        return $out;
    }

    
    /**
     * Determine if value is within a array, and 
     * provide location index if found.
     * 
     * @param string  $strNeedle  String value to find in Haystack
     * @param array   $aHaystack  Haystack of values
     * @param integer &$iIndexRef Index of where needle was found
     * @param boolean $bSort      Sort provided array before finding index.
     * 
     * @return boolean True if found, else False
     */
    public static function arrayIndex(
        $strNeedle,
        $aHaystack,
        &$iIndexRef = null,
        $bSort = false
    ) {
        if ( $bSort ) {
            sort($aHaystack);
        }
        
        for ( $iIndex=0; $iIndex< count($aHaystack); $iIndex++ ) {
            if ($aHaystack[$iIndex] == $strNeedle) {
                if ( !is_null($iIndexRef) ) {
                    $iIndexRef = $iIndex;
                }
                return true;
            }
        }
        
        return false;
    }
    
    /**
     * Sort and Enumerate an array
     * 
     * @param array  $aValues Array of values to sort and enumerated
     * @param string $strGlue Glue delimiting values.
     * 
     * @return string Enumerated sorted array values
     */
    public static function arrayEnum(
        $aValues,
        $strGlue = "\n"
    ) {
        $funcEnum = create_function('$k,$v,$g', 'return $k+1 . " = " . $v . $g;');
        if (!$funcEnum) {
            throw new RuntimeException( "Failed to create anonymous function");
        }
        
        sort($aValues, SORT_STRING);
        
        $strEnum = "";
        foreach ($aValues as $key => $value) {
            $strEnum .= $funcEnum($key, $value, $strGlue);
        }
        return rtrim($strEnum, $strGlue);
    }
    
    /**
     * Get local environment value
     * 
     * @param string $strEnvKey Environment key
     * 
     * @return mixed Environment value, else False if not set
     */
    public static function getEnvValue($strEnvKey)
    {
        if ( is_null($strEnvKey) 
            || !is_string($strEnvKey)
            || empty($strEnvKey)
        ) {
            return false;
        }
        
        return getenv($strEnvKey);
    }

    /**
     * Replace line when pattern (RegExp) is found.
     * 
     * @param string $strFilePath File path
     * @param string $strRegExp   RegExp pattern to match
     * @param string $strReplace  Replacement line where pattern match if found.
     * 
     * @return boolean True if found, else False
     */
    public static function replaceLine($strFilePath, $strRegExp, $strReplace)
    {
        if (!file_exists($strFilePath)) {
            throw new InvalidArgumentException( "File \"{$strFilePath}\" not exists");
        }

        if ( ($aLines = file($strFilePath, FILE_SKIP_EMPTY_LINES | FILE_TEXT)) === false ) {
            throw new RuntimeException( "Failed to read File \"{$strFilePath}\"");
        }
        
        $strContent = "";
        $bFound = false;
        
        for ($i = 0; $i < count($aLines); $i++) {
            if (!$bFound 
                && preg_match($strRegExp, $aLines[$i])
            ) {
                $strContent .= $strReplace . "\n";
                $bFound = true;
            } else { // the
                $strContent .= $aLines[$i];
            }
        }
        
        if ( ($pFile = fopen($strFilePath, "w")) === false ) {
            throw new RuntimeException( "Failed to open File \"{$strFilePath}\"");
        }
        fwrite($pFile, $strContent);
        fclose($pFile);
        
        return $bFound;
    }
    
    /**
     * Match line pattern (RegExp) is found.
     * 
     * @param string  $strFilePath       File path
     * @param string  $strRegExp         RegExp pattern to match
     * @param boolean $bStripWhiteSpaces Strip Whitespaces
     * 
     * @return boolean True if found, else False
     */
    public static function matchLine(
        $strFilePath, 
        $strRegExp,
        $bStripWhiteSpaces = false
    ) {
        if (!file_exists($strFilePath)) {
            throw new InvalidArgumentException( "File \"{$strFilePath}\" not exists");
        }

        if ( ($aLines = file($strFilePath, FILE_SKIP_EMPTY_LINES | FILE_TEXT)) === false ) {
            throw new RuntimeException( "Failed to read File \"{$strFilePath}\"");
        }
        
        for ($i = 0; $i < count($aLines); $i++) {
            $strLine = $aLines[$i];
            if ($bStripWhiteSpaces) {
                $strLine = preg_replace('/\s\s*/', '', $strLine);    
            }
            if (preg_match($strRegExp, $strLine) ) {
                return true;
            }
        }
        
        return false;
    }

    /**
     * Multibyte match line pattern (RegExp) is found.
     * 
     * @param string $strFilePath File path
     * @param string $strRegExp   RegExp pattern to match
     * 
     * @return boolean True if found, else False
     */
    public static function matchLineMb(
        $strFilePath, 
        $strRegExp
    ) {
        if (!file_exists($strFilePath)) {
            throw new InvalidArgumentException( "File \"{$strFilePath}\" not exists");
        }

        if ( ($aLines = file($strFilePath, FILE_SKIP_EMPTY_LINES | FILE_TEXT)) === false ) {
            throw new RuntimeException( "Failed to read File \"{$strFilePath}\"");
        }
        
        for ($i = 0; $i < count($aLines); $i++) {
            $strLine = $aLines[$i];
            $strLine = mb_ereg_replace('[^a-zA-Z0-9\.\-\_]', '', $strLine);
            if ( mb_ereg_match($strRegExp, $strLine) ) {
                return true;
            }
        }
        
        return false;
    }
    
    
    /**
     * Determine if two directories are the same.
     * 
     * @param string $strDirA Directory path
     * @param string $strDirB Directory path
     * 
     * @return boolean Returns TRUE if same directory, else FALSE
     */
    public static function isDirSame( $strDirA, $strDirB )
    {
        if ( !is_dir($strDirA) ) {
            throw new InvalidArgumentException( "Directory \"{$strDirA}\" not exists");
        }
        if ( !is_dir($strDirB) ) {
            throw new InvalidArgumentException( "Directory \"{$strDirB}\" not exists");
        }
        
        $strRealDirA = realpath($strDirA);
        $strRealDirB = realpath($strDirB);
        
        return strcasecmp($strRealDirA, $strRealDirB) == 0;
    }

    /**
     * Copy directory
     * 
     * @param string  $strSourceDir     Source
     * @param string  $strTargetDir     Target
     * @param boolean $bRemoveTargetDir Remove Directory
     * @param boolean $bCopyContents    Copy Source contents
     * 
     * @return boolean Return TRUE upon success, else FALSE
     */
    public static function doCopyDir( 
    	$strSourceDir,
    	$strTargetDir,
    	$bRemoveTargetDir = false,
    	$bCopyContents = false
    ) {
        if ( !is_dir($strSourceDir) ) {
            throw new InvalidArgumentException( 
            	"Source directory \"{$strSourceDir}\" does not exist"
           	);
        }
        
        if ( $bCopyContents ) {
        	$strSourceDir = rtrim($strSourceDir, "\/\\ ") . "\\*";
        }
        
        if (is_dir($strTargetDir)) {
			if ( $bRemoveTargetDir ) {
	            self::doRemoveDir($strTargetDir);
	        }
        }
        
        $strTargetDir = rtrim($strTargetDir, "\/\\ ") . "\\";
        $strExecCmd 
        	= "XCOPY /E /Q /V \"{$strSourceDir}\" \"{$strTargetDir}\"";

        exec($strExecCmd);
        chmod($strTargetDir, 0777);
            
        return true;
    }
    
    /**
     * Clear all files from directory
     * 
     * @param string $strCleanDir Directory to Clean
     * 
     * @return boolean Return TRUE upon success, else FALSE
     */
    public static function doCleanDir( $strCleanDir )
    {
        if ( !is_dir($strCleanDir) ) {
            throw new InvalidArgumentException( 
            	"Directory \"{$strCleanDir}\" does not exist"
            );
        }
        
        $strCurrentDir = getcwd();
        chdir($strCleanDir);
        exec("del *.* /s /q");
        chdir($strCurrentDir);
            
        return true;
    }
    
    /**
     * Remove directory
     * 
     * @param string $strRemoveDir Directory to Remove
     * 
     * @return boolean Return TRUE upon success, else FALSE
     */
    public static function doRemoveDir( $strRemoveDir )
    {        
        if ( !is_dir($strRemoveDir) ) {
            return true;
        }
        
        $strBaseDir = dirname($strRemoveDir);
        $strDirName = basename($strRemoveDir);
        
        $strCurrentDir = getcwd();
        chdir($strBaseDir);
        exec("RMDIR /S /Q \"{$strDirName}\"");
        chdir($strCurrentDir);
        
        return true;
    }
    
    /**
     * Create directory
     * 
     * @param string $strCreateDir Directory to Create
     * 
     * @return boolean Return TRUE upon success, else FALSE
     */
    public static function doCreateDir( $strCreateDir )
    {        
        exec("mkdir {$strCreateDir}");
        
        return true;
    }
    
    /**
     * Remove file
     * 
     * @param string $strRemoveFile File to remove
     * 
     * @return boolean Return TRUE upon success, else FALSE
     */
    public static function doRemoveFile( $strRemoveFile )
    {
        if ( !is_file($strRemoveFile) ) {
            return true;
        }
        
        $strBaseDir = dirname($strRemoveFile);
        $strFileName = basename($strRemoveFile);
        
        $strCurrentDir = getcwd();
        chdir($strBaseDir);
        $strCommand = "del \"{$strFileName}\" /s /q";

        exec($strCommand);
        chdir($strCurrentDir);
            
        sleep(1);
        
        if ( is_file($strRemoveFile) ) {
            throw new RuntimeException("Failed to delete file \"{$strRemoveFile}\"");
        }
                 
        return true;
    }
    
    /**
     * Open default browser to provided URL
     * 
     * @param string $strURL        URL expected to view
     * @param string &$strResultMsg Result
     * @param string &$strOutputMsg Details
     * 
     * @return void
     */
    public static function doOpenDefaultBrowser(
    	$strURL,
    	&$strResultMsg,
    	&$strOutputMsg
    ) {
    	$strCommand = "start {$strURL}";
		
		self::doExecute($strCommand, $strResultMsg, $strOutputMsg);
    }
    
    /**
     * Execute command
     * 
     * @param string $strCommand    Command
     * @param string &$strResultMsg Result
     * @param string &$strOutputMsg Details
     * 
     * @return void
     */
    public static function doExecute(
    	$strCommand,
    	&$strResultMsg,
    	&$strOutputMsg
    ) {
		$aOutput = array();
		$strResultMsg = exec($strCommand, $aOutput);
		
		$strOutputMsg = "";
		$bFirstLine = true;
		foreach ($aOutput as $strMsg) {
			if (empty($strMsg)) {
				continue;
			}
			
			if (!$bFirstLine) {
				$strOutputMsg .= "          ";	
			} else {
				$bFirstLine = false;
			}
			
			$strOutputMsg .= wordwrap($strMsg, 60, "\n          ") . "\n";
		}    	
    }
    /**
     * Download blob from Windows Azure Blob storage using
     * standard PHP file IO functions.
     * 
     * @param string  $strBlobStorageURL   Blob name to download
     * @param string  $strTargetPath       Target path where to place download
     * @param integer $iExpectedSizeKB     Expected Size of ZIP resource
     * @param boolean $bIfFailedDie        Die upon failure
     * @param boolean &$bNetworkConnection TRUE if able to establish connection, else FALSE 
     * 
     * @return boolean Returns TRUE on success or FALSE on failure
     */
    public static function doCopyRemoteFile(
        $strBlobStorageURL, 
        $strTargetPath,
        $iExpectedSizeKB,
        $bIfFailedDie,
        &$bNetworkConnection
    ) {
        $bSuccess 				= false; 
		$bCurlSuccess 			= true;
		$strErrMsg 				= null;
		$bNetworkConnection 	= false;
		$bNoConnection  		= false;
		
		if ( is_null($strTargetPath)
			|| !is_string($strTargetPath)
			|| empty($strTargetPath)
		) {
			throw new InvalidArgumentException("Not defined: \"strTargetPath\"");
		}
    	
		if ( is_null($strBlobStorageURL)
			|| !is_string($strBlobStorageURL)
			|| empty($strBlobStorageURL)
		) {
			throw new InvalidArgumentException("Not defined: \"strBlobStorageURL\"");
		}
			     
    	try
    	{
    		if ( is_file($strTargetPath) ) {
    			unlink($strTargetPath);
    		}
    		
	        if ( !($fpWrite = fopen($strTargetPath, "w+b")) ) {
	        	throw new RuntimeException( "Failed create handle for target file \"{$strTargetPath}\"");
	        }
	        
	        $bNetworkConnection = @copy($strBlobStorageURL, $strTargetPath);
	
	        fclose($fpWrite);
	        	    	
			if ( !$bNetworkConnection 
				|| !is_file($strTargetPath)
			) {
	        	return false;
	        }
	        
	        $iActualSizeDB = floor(filesize($strTargetPath) / 1024);
	        $iExpectedSizeKB = floor($iExpectedSizeKB * 0.9);
	        if ( $iActualSizeDB == 0 || $iActualSizeDB < $iExpectedSizeKB ) {
	        	return false;
	        }
	        
	        $bSuccess = true;
    	}
    	catch ( RuntimeException $ex )
    	{
    		throw $ex;
    	}
        catch ( Exception $ex ) 
        {
            throw new RuntimeException( 
                "Exception: \""
                . $ex->getMessage()                    
                . "\"" 
            );	    		
        }

        return $bSuccess;	
    }

    /**
     * WMIC: Determine if a process is currently running
     * 
     * @param string $strProcessName Name of process to 
     * @param string $strTargetDir   Target directory to hold process list
     * 
     * @return boolean Returns TRUE if started, else FALSE 
     */    
    public static function isProcessRunning(
        $strProcessName,
        $strTargetDir
    ) {    
        if ( !is_dir($strTargetDir) ) {
            throw new RuntimeException( "Target directory \"{$strTargetDir}\" does not exist");
        }
        
        $strProcessListFile  = rtrim($strTargetDir, "\/\\ ");
        $strProcessListFile .= "/";
        $strProcessListFile .= WACmdLineTools_Util_Constants::FILE_PROCESS_LIST;
        $strProcessListFile = WACmdLineTools_Util::doPathTransform($strProcessListFile);
        
        if ( is_file($strProcessListFile) ) {
            unlink($strProcessListFile);
        }
            
        $strCommand = "WMIC /OUTPUT:\"{$strProcessListFile}\" PROCESS get Caption";
        $aOutput = array();
        $strResult = exec($strCommand, $aOutput);
        
        sleep(1);
        
        if ( !is_file($strProcessListFile) ) {
            throw new RuntimeException( "Failed to create " . WACmdLineTools_Util_Constants::FILE_PROCESS_LIST . "");
        }
        
        $strRegExp = '/' . $strProcessName . '/';    
        $bFound 
            = WACmdLineTools_Util::matchLineMb(
                $strProcessListFile, 
                $strProcessName
            );

        return $bFound;    
    }
    
}
?>