<?php
/**
 * Zip.php
 * 
 * Support for handling ZIP files
 * 
 * PHP Version 5
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *       Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *       
 *       Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *       
 *       Neither the name of Microsoft nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Microsoft ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Microsoft BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */

/**
 * Abstract base class
 *
 * @category  Microsoft
 * @package   WindowsAzureCmdLineTools4PHP
 * @author    Jeff Tanner <a-jetann@microsoft.com>
 * @copyright 2010 Copyright  Microsoft Corporation. All Rights Reserved
 * @license   New BSD License (BSD), http://azurephptools.codeplex.com/license
 * @link      http://www.microsoft.com
 */
abstract class WACmdLineTools_Util_Zip
{
    /**
     * Constructor
     * 
     * @return void
     */
    function __construct()
    {
        $this->init();
    }

    /**
     * Initialize ZIP support, load "zip" extension
     * 
     * @return boolean Returns TRUE on success or FALSE on failure
     */
    private function _init()
    {
        if (!extension_loaded('zip') ) {
            if ( !dl('php_zip.dll') ) {
                throw new RuntimeException( "Failed to load \"zip\" extension");
            }
        }

        return true;
    }
       
    /**
     * Extract all from ZIP file
     * 
     * @param string $strZipFilePath ZIP file path
     * @param string $strTargetPath  ZIP file contents destination
     * 
     * @return boolean Returns TRUE on success or FALSE on failure
     */
    public static function zipExtractAll(
        $strZipFilePath, 
        $strTargetPath
    ) {
        return self::zipExtractContents($strZipFilePath, $strTargetPath);
    }

    /**
     * Extract on a specific folder from ZIP file
     * 
     * @param string $strZipFilePath ZIP file path
     * @param string $strTargetPath  ZIP file contents destination
     * @param string $strZipFolder   Folder to Extract from ZIP file
     * 
     * @return boolean Returns TRUE on success or FALSE on failure
     */
    public static function zipExtractFolder(
        $strZipFilePath, 
        $strTargetPath, 
        $strZipFolder
    ) {
        $strZipFileName = basename($strZipFilePath);
           
        $bSuccess = false; 
        if ( !is_file($strZipFilePath) ) {
            throw new InvalidArgumentException( "\"{$strZipFilePath}\" is not a valid file" );
        }
        
        $ext = WACmdLineTools_Util::getFileExtention($strZipFilePath);
        if ($ext != "zip") {
            throw new InvalidArgumentException( "\"{$strZipFilePath}\" is not a ZIP file" );
        }
        
        try {  
	        $files = array();
	        $zip = new ZipArchive;
	        if ($zip->open($strZipFilePath) === true) {
	            for ($i = 0; $i < $zip->numFiles; $i++) {
	                $strEntry = $zip->getNameIndex($i);
			
	                // Use strpos() to check if the entry name 
	                // contains the directory we want to extract   
	                if ( strpos(str_replace("\\", "/", $strEntry), str_replace("\\", "/", $strZipFolder)) === false ) {
	                    continue;
	                }
	                // Add the entry to our array if it is in our desired directory
	                $files[] = $strEntry;
	            }
	            
	            if (empty($files)) {
	                throw new RuntimeException( 
	                	"No files to extract from folder "
	                	. "\"{$strZipFolder}\" within "
	                	. "ZIP file \"{$strZipFilePath}\"" 
	                );
	            }
	            
	            // Feed $files array to extractTo() to get only the files we want
	            if ($zip->extractTo($strTargetPath, $files) !== true) {
	                throw new RuntimeException( 
	                    "Failed to extract folder \"{$strZipFolder}\""
	                    . " from ZIP file \"{$strZipFilePath}\"" 
	                );
	            }
	               
	            $zip->close();
	            
	            usleep(1000);
	           
	            $bSuccess = true;
	       	} else {
	            throw new RuntimeException( 
	                "Failed to open ZIP file \"{$strZipFileName}\"" 
	            );
			}
        }
        catch ( Exception $ex ) 
        {
            throw $ex;	    		
        }
        
        return $bSuccess;
    }
    
    /**
     * Extract on a specific folder from ZIP file
     * 
     * @param string $strZipFilePath ZIP file path
     * @param string $strTargetPath  ZIP file contents destination
     * @param mixed  $mixContents    Contents to Extract from ZIP file
     * 
     * @return boolean Returns TRUE on success or FALSE on failure
     */
    public static function zipExtractContents(
        $strZipFilePath, 
        $strTargetPath, 
        $mixContents = null
    ) {
        
        $strContents = is_null($mixContents) 
            ? "all" 
            : ( is_array($mixContents) ? implode(", ", $mixContents) : $mixContents );
        $strZipFileName = basename($strZipFilePath);
           
           $bSuccess = false; 
        if ( !is_file($strZipFilePath) ) {
               throw new InvalidArgumentException( "\"{$strZipFilePath}\" is not a valid file" );
        }
        $ext = WACmdLineTools_Util::getFileExtention($strZipFilePath);
        if ($ext != "zip") {
            throw new InvalidArgumentException( "\"{$strZipFilePath}\" is not a ZIP file" );
        }
        if ( !is_null($mixContents) 
            && !( is_string($mixContents) && !empty($mixContents) )
            && !( is_array($mixContents) && !empty($mixContents) )
        ) {
            throw new InvalidArgumentException(
            	"Invalid contents request \"{$mixContents}\""
                . " from ZIP file \"{$strZipFilePath}\"" 
            );
        }
        
        try {             
	        $files = array();
	        $zip = new ZipArchive;
	        $mixResult = $zip->open($strZipFilePath);
	        if ($mixResult === true) {     
	            // Feed $files array to extractTo() to get only the files we want
	            if ($zip->extractTo($strTargetPath, $mixContents) !== true) {
	                throw new RuntimeException( 
	                    "Failed to extract contents \"{$strContents}\""
	                    . " from ZIP file \"{$strZipFilePath}\"" 
	                );
	            }
	            
	            $zip->close();
	            
	            $bSuccess = true;
	        } else {
	            throw new RuntimeException (
	                "Failed to open"
	                . " ZIP file \"{$strZipFileName}\""
	                . PHP_EOL
	                . self::_zipFileErrMsg($mixResult)
	            );
	        }
		}
		catch ( Exception $ex ) 
		{
			throw $ex;	    		
		}
        
        return $bSuccess;
    }    
    /**
     * Get ZIP error message.
     * 
     * @param int $errno ZIP error
     * 
     * @return string ZIP Error Message
     */
    private static function _zipFileErrMsg($errno) 
    {
        // using constant name as a string to make this function PHP4 compatible
        $zipFileFunctionsErrors = array(
            'ZIPARCHIVE::ER_MULTIDISK' => 'Multi-disk zip archives not supported',
            'ZIPARCHIVE::ER_RENAME' => 'Renaming temporary file failed',
            'ZIPARCHIVE::ER_CLOSE' => 'Closing zip archive failed', 
            'ZIPARCHIVE::ER_SEEK' => 'Seek error',
            'ZIPARCHIVE::ER_READ' => 'Read error',
            'ZIPARCHIVE::ER_WRITE' => 'Write error',
            'ZIPARCHIVE::ER_CRC' => 'CRC error',
            'ZIPARCHIVE::ER_ZIPCLOSED' => 'Containing zip archive was closed',
            'ZIPARCHIVE::ER_NOENT' => 'No such file.',
            'ZIPARCHIVE::ER_EXISTS' => 'File already exists',
            'ZIPARCHIVE::ER_OPEN' => 'Can\'t open file', 
            'ZIPARCHIVE::ER_TMPOPEN' => 'Failure to create temporary file.', 
            'ZIPARCHIVE::ER_ZLIB' => 'Zlib error',
            'ZIPARCHIVE::ER_MEMORY' => 'Memory allocation failure', 
            'ZIPARCHIVE::ER_CHANGED' => 'Entry has been changed',
            'ZIPARCHIVE::ER_COMPNOTSUPP' => 'Compression method not supported', 
            'ZIPARCHIVE::ER_EOF' => 'Premature EOF',
            'ZIPARCHIVE::ER_INVAL' => 'Invalid argument',
            'ZIPARCHIVE::ER_NOZIP' => 'Not a zip archive',
            'ZIPARCHIVE::ER_INTERNAL' => 'Internal error',
            'ZIPARCHIVE::ER_INCONS' => 'Zip archive inconsistent', 
            'ZIPARCHIVE::ER_REMOVE' => 'Can\'t remove file',
            'ZIPARCHIVE::ER_DELETED' => 'Entry has been deleted',
        );
        $errmsg = 'unknown';
        foreach ($zipFileFunctionsErrors as $constName => $errorMessage) {
            if (defined($constName) and constant($constName) === $errno) {
                return "Zip File Function error: \"{$errorMessage}\"";
            }
        }
        
        return 'Zip File Function error: unknown';
    }
    
}
